# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
This file is part of the privacyidea service
'''
import logging

from pylons import request, response, config, tmpl_context as c


from privacyidea.lib.base import BaseController

from privacyidea.lib.token import enableToken, assignToken
from privacyidea.lib.token import unassignToken, removeToken
from privacyidea.lib.token import setPin, setMaxFailCount, setOtpLen
from privacyidea.lib.token import setSyncWindow, setCounterWindow
from privacyidea.lib.token import setDescription
from privacyidea.lib.token import resyncToken, resetToken, setPinUser
from privacyidea.lib.token import setPinSo, setHashLib, addTokenInfo
from privacyidea.lib.token import TokenIterator, initToken, setRealms
from privacyidea.lib.token import getTokenType, get_serial_by_otp
from privacyidea.lib.token import getTokens4UserOrSerial, copyTokenPin
from privacyidea.lib.token import copyTokenUser, losttoken, check_serial
from privacyidea.lib.token import getTokenRealms
from privacyidea.lib.token import genSerial
from privacyidea.lib.token import newToken
from privacyidea.lib.token import get_token_type_list

from privacyidea.lib.error import ParameterError
from privacyidea.lib.util import getParam, getLowerParams
from privacyidea.weblib.util import get_client
from privacyidea.lib.util import remove_session_from_param
from privacyidea.lib.user import getSearchFields, getUserList, User
from privacyidea.lib.user import getUserFromParam, getUserFromRequest

from privacyidea.lib.realm import getDefaultRealm

from privacyidea.lib.reply import sendResult, sendError, sendXMLResult
from privacyidea.lib.reply import sendXMLError, sendCSVResult
from privacyidea.lib.reply import sendQRImageResult

from privacyidea.lib.validate import get_challenges

from privacyidea.model.meta import Session
from privacyidea.lib.policy import PolicyClass
from privacyidea.lib.policy import PolicyException
from privacyidea.lib.audit import logTokenNum
from privacyidea.lib.config import get_privacyIDEA_config
# for loading XML file
from privacyidea.lib.ImportOTP import parseSafeNetXML, parseOATHcsv
from privacyidea.lib.ImportOTP import ImportException, parseYubicoCSV


from tempfile import mkstemp
import os
import traceback
import webob

from privacyidea.lib.log import log_with


log = logging.getLogger(__name__)

optional = True
required = False


class AdminController(BaseController):

    '''
    The privacyidea.controllers are the implementation of the web-API to talk
    to the privacyIDEA server.
    The AdminController is used for administrative tasks like adding tokens
    to privacyIDEA,
    assigning tokens or revoking tokens.
    The functions of the AdminController are invoked like this

        https://server/admin/<functionname>

    The functions are described below in more detail.
    '''

    @log_with(log)
    def __before__(self, action, **params):
        '''
        '''
        try:
            c.audit['success'] = False
            c.audit['client'] = get_client()
            self.Policy = PolicyClass(request, config, c,
                                      get_privacyIDEA_config(),
                                      tokenrealms=request.params.get('serial'),
                                      token_type_list=get_token_type_list())
            self.set_language()

            self.before_identity_check(action)

            Session.commit()
            return request

        except webob.exc.HTTPUnauthorized as acc:  # pragma: no cover
            # the exception, when an abort() is called if forwarded
            log.info("%r: webob.exception %r" % (action, acc))
            log.info(traceback.format_exc())
            Session.rollback()
            Session.close()
            raise acc

        except Exception as exx:  # pragma: no cover
            log.error("exception %r" % (action, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, exx, context='before')

        finally:
            pass

    @log_with(log)
    def __after__(self, action, **params):
        '''
        '''
        params = {}

        try:
            params.update(request.params)
            c.audit['administrator'] = getUserFromRequest(request).get("login")
            if 'serial' in params:
                    c.audit['serial'] = request.params['serial']
                    c.audit['token_type'] = getTokenType(params.get('serial'))

            self.audit.log(c.audit)

            Session.commit()
            return request

        except Exception as e:  # pragma: no cover
            log.error("unable to create a session cookie: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, context='after')

        finally:
            Session.close()

    @log_with(log)
    def show(self, action, **params):
        """
        method:
            admin/show

        description:
            displays the list of the available tokens

        arguments:
            * serial  - optional: only this serial will be displayed
            * user    - optional: only the tokens of this user will be
                                  displayed. If the user does not exist,
                                  privacyidea will search tokens of users,
                                  who contain this substring.
              **TODO:** This can be very time consuming and will be changed
                        in the next release to use wildcards.
            * filter  - optional: takes a substring to search in table token
                                  columns
            * viewrealm - optional: takes a realm, only the tokens in this
                                    realm will be displayed
            * sortby  - optional: sort the output by column
            * sortdir - optional: asc/desc
            * page    - optional: reqeuest a certain page
            * pagesize- optional: limit the number of returned tokens
            * user_fields - optional: additional user fields from the userid
                                      resolver of the owner (user)
            * outform - optional: if set to "csv", than the token list will be
                                  given in CSV

        returns:
            a json result with:
            { "head": [],
            "data": [ [row1], [row2] .. ]
            }

        exception:
            if an error occurs an exception is serialized and returned
        """

        param = request.params
        try:
            serial = getParam(param, "serial", optional)
            page = getParam(param, "page", optional)
            filt = getParam(param, "filter", optional)
            sort = getParam(param, "sortby", optional)
            sdir = getParam(param, "sortdir", optional)
            psize = getParam(param, "pagesize", optional)
            realm = getParam(param, "viewrealm", optional)
            ufields = getParam(param, "user_fields", optional)
            output_format = getParam(param, "outform", optional)

            user_fields = []
            if ufields:
                user_fields = [u.strip() for u in ufields.split(",")]

            user = getUserFromParam(param, optional)

            filterRealm = []
            # check admin authorization
            res = self.Policy.checkPolicyPre('admin', 'show', param, user=user)

            filterRealm = res['realms']
            # check if policies are active at all
            # If they are not active, we are allowed to SHOW any tokens.
            pol = self.Policy.getAdminPolicies("show")
            # If there are no admin policies, we are allowed to see all realms
            if not pol['active']:
                filterRealm = ["*"]

            # If the admin wants to see only one realm, then do it:
            log.debug("[checking to only see tokens in realm <%s>" % realm)
            if realm:
                if realm in filterRealm or '*' in filterRealm:
                    filterRealm = [realm]

            log.info("admin >%s< may display the following realms: %s" %
                     (res['admin'], filterRealm))
            log.debug("displaying tokens: serial: %s, page: %s, filter: %s,"
                      " user: %s", serial, page, filt, user.login)

            toks = TokenIterator(user, serial, page, psize, filt,
                                 sort, sdir, filterRealm, user_fields)

            c.audit['success'] = True
            c.audit['info'] = "realm: %s, filter: %r" % (filterRealm, filt)

            # put in the result
            result = {}

            # now row by row
            lines = []
            for tok in toks:
                # CKO:
                log.debug("tokenline: %s" % tok)
                lines.append(tok)

            result["data"] = lines
            result["resultset"] = toks.getResultSetInfo()

            Session.commit()

            if output_format == "csv":
                return sendCSVResult(response, result)
            else:
                return sendResult(response, result)

        except PolicyException as pe:
            log.error('policy failed: %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:  # pragma: no cover
            log.error('failed: %r' % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()

########################################################
    @log_with(log)
    def remove(self, action, **params):
        """
        method:
            admin/remove

        description:
            deletes either a certain token given by serial or all tokens
            of a user

        arguments:
            * serial  - optional
            * user    - optional

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params

        try:
            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'remove', param)

            log.info("removing token with serial %s for user %s" %
                     (serial, user.login))
            ret = removeToken(user, serial)

            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            logTokenNum()
            c.audit['success'] = ret

            Session.commit()
            return sendResult(response, ret)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:  # pragma: no cover
            log.error("failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def enable(self, action, **params):
        """
        method:
            admin/enable

        description:
            enables a token or all tokens of a user

        arguments:
            * serial  - optional
            * user    - optional

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params
        try:
            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            # check admin authorization
            # Determine the token realm, the token is in.
            tokenrealms = []
            if serial:
                tokenrealms = getTokenRealms(serial)
            self.Policy.checkPolicyPre('admin', 'enable', param, user=user,
                                       tokenrealms=tokenrealms)
            log.info("enable token with serial %s for user %s@%s.",
                     serial, user.login, user.realm)
            ret = enableToken(True, user, serial)

            c.audit['success'] = ret
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            logTokenNum()
            Session.commit()
            return sendResult(response, ret, 1)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:  # pragma: no cover
            log.error("failed: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            
            return sendError(response, e, 1)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getSerialByOtp(self, action, **params):
        """
        method:
            admin/getSerialByOtp

        description:
            searches for the token, that generates the given OTP value.
            The search can be restricted by several critterions

        arguments:
            * otp      - required. Will search for the token, that produces
                                   this OTP value
            * type     - optional, will only search in tokens of type
            * realm    - optional, only search in this realm
            * assigned - optional. 1: only search assigned tokens, 0: only
                                   search unassigned tokens

        returns:
            a json result with the serial


        exception:
            if an error occurs an exception is serialized and returned

        """
        ret = {}
        param = request.params

        try:
            otp = getParam(param, "otp", required)
            typ = getParam(param, "type", optional)
            realm = getParam(param, "realm", optional)
            assigned = getParam(param, "assigned", optional)

            serial = ""
            username = ""

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'getserial', param)

            serial, username, resClass = get_serial_by_otp(None, otp, 10,
                                                           typ=typ,
                                                           realm=realm,
                                                           assigned=assigned)
            log.debug("found %s with user %s" % (serial, username))

            if "" != serial:
                self.Policy.checkPolicyPost('admin',
                                            'getserial',
                                            {'serial': serial})

            c.audit['success'] = 1
            c.audit['serial'] = serial

            ret['success'] = True
            ret['serial'] = serial
            ret['user_login'] = username
            ret['user_resolver'] = resClass

            Session.commit()
            return sendResult(response, ret, 1)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            c.audit['success'] = 0
            Session.rollback()
            log.error('error: %r' % e)
            log.error(traceback.format_exc())
            return sendError(response, e, 1)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def disable(self, action, **params):
        """
        method:
            admin/disable

        description:
            disables a token given by serial or all tokens of a user

        arguments:
            * serial  - optional
            * user    - optional

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params
        try:
            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'disable', param, user=user)

            log.info("disable token with serial %s for user %s@%s.",
                     serial, user.login, user.realm)
            ret = enableToken(False, user, serial)

            c.audit['success'] = ret
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            Session.commit()
            return sendResult(response, ret, 1)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, 1)

        finally:
            Session.close()


#######################################################
    @log_with(log)
    def check_serial(self, action, **params):
        '''
        method
            admin/check_serial

        description:
            This function checks, if a given serial will be unique.
            It returns True if the serial does not yet exist and
            new_serial as a new value for a serial, that does not exist, yet

        arguments:
            serial    - required- the serial to be checked

        returns:
            a json result with a new suggestion for the serial

        exception:
            if an error occurs an exception is serialized and returned

        '''
        param = request.params
        try:
            serial = getParam(param, "serial", required)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'checkserial', param)

            log.info("checking serial %s" % serial)
            (unique, new_serial) = check_serial(serial)

            c.audit['success'] = True
            c.audit['serial'] = serial
            c.audit['action_detail'] = "%r - %r" % (unique, new_serial)

            Session.commit()
            return sendResult(response, {"unique": unique,
                                         "new_serial": new_serial}, 1)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log, log_entry=False)
    def init(self, action, **params):
        """
        method:
            admin/init

        description:
            creates a new token.

        arguments:
            * otpkey    - required - the hmac Key of the token
            * genkey    - required - =1, if key should be generated. We either
                                    need otpkey or genkey
            * keysize   - optional - either 20 or 32. Default is 20
            * serial    - required - the serial number/identifier of the token
            * description - optional
            * pin        - optional - the pin of the user pass
            * user       - optional - login user name
            * realm      - optional - realm of the user
            * type       - optional - the type of the token
            * tokenrealm - optional - the realm a token should be put into
            * otplen     - optional  - length of the OTP value
            * hashlib    - optional  - used hashlib sha1 oder sha256

        ocra arguments:
            for generating OCRA Tokens type=ocra you can specify the following
                                    parameters:
            * ocrasuite    - optional - if you do not want to use the default
                                    ocra suite OCRA-1:HOTP-SHA256-8:QA64
            * sharedsecret - optional - if you are in Step0 of enrolling an
                                    OCRA/QR token the
              sharedsecret=1 specifies,
              that you want to generate a shared secret
            * activationcode - optional - if you are in Step1 of enrolling an
                                    OCRA token you need to pass the
              activation code, that was generated in the QRTAN-App

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        ret = False
        response_detail = {}
        helper_param = {}

        try:
            tokenrealm = None
            param = request.params
            helper_param.update(param)

            user = getUserFromParam(param, optional)

            # check admin authorization
            user_tnum = len(getTokens4UserOrSerial(user))
            res = self.Policy.checkPolicyPre('admin', 'init', param, user=user,
                                             options={'token_num': user_tnum})

            if user is not None:
                helper_param['user.login'] = user.login
                helper_param['user.realm'] = user.realm

            # for genkey, we have to transfer this to the lowest level
            key_size = getParam(param, "keysize", optional) or 20
            helper_param['key_size'] = key_size

            tok_type = getParam(param, "type", optional) or 'hmac'

            # if no user is given, we put the token in all realms of the admin
            if user.login == "":
                log.debug("setting tokenrealm %s" % res['realms'])
                tokenrealm = res['realms']

            # look for the tokenclass to support a class init
            # the classInit could do a rewrite of the request parameters
            # which are then used in the tokenInit as parameters
            # this is for example
            #   to find all open init challenges of a token type and set the
            #   serial number in the parameter list

            g = config['pylons.app_globals']
            tokenclasses = g.tokenclasses

            tokenTypes = tokenclasses.keys()
            if tok_type in tokenTypes:
                tclass = tokenclasses.get(tok_type)
                tclass_object = newToken(tclass)
                if hasattr(tclass_object, 'classInit'):
                    h_params = tclass_object.classInit(param, user=user)
                    helper_param.update(h_params)

            serial = helper_param.get('serial', None)
            prefix = helper_param.get('prefix', None)
            if serial is None or len(serial) == 0:
                serial = genSerial(tok_type, prefix)
                helper_param['serial'] = serial

            log.info("initialize token. user: %s, serial: %s" % (user.login,
                                                                 serial))
            (ret, tokenObj) = initToken(helper_param,
                                        user,
                                        tokenrealm=tokenrealm)

            # result enrichment - if the token is sucessfully created,
            # some processing info is added to the result document,
            #  e.g. the otpkey :-) as qr code
            initDetail = tokenObj.getInitDetail(helper_param, user)
            response_detail.update(initDetail)

            c.audit['success'] = ret
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            c.audit['serial'] = serial
            c.audit['token_type'] = tok_type

            logTokenNum()

            # setting the random PIN
            randomPINLength = self.Policy.getRandomOTPPINLength(user)
            if randomPINLength > 0:
                newpin = self.Policy.getRandomPin(randomPINLength)
                log.debug("setting random pin for token with serial "
                          "%s and user: %s" % (serial, user))
                setPin(newpin, None, serial)
                
            c.audit['success'] = ret

            Session.commit()

            # finally we render the info as qr immage, if the qr parameter
            # is provided and if the token supports this
            if 'qr' in param and tokenObj is not None:
                (rdata, hparam) = tokenObj.getQRImageData(response_detail)
                hparam.update(response_detail)
                hparam['qr'] = param.get('qr') or 'html'
                return sendQRImageResult(response, rdata, hparam)
            else:
                return sendResult(response, ret, opt=response_detail)

        except PolicyException as pe:
            log.error("policy failed %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("token initialization failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def unassign(self, action, **params):
        """
        method:
            admin/unassign - remove the assigned user from the token

        description:
            unassigns a token from a user. i.e. the binding between the token
            and the user is removed

        arguments:
            * serial    - required - the serial number/identifier of the token
            * user      - optional

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params

        try:

            serial = getParam(param, "serial", required)
            user = getUserFromParam(param, optional)
            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'unassign', param)
            log.info("unassigning token with serial %r from "
                     "user %r@%r" % (serial, user.login, user.realm))
            res = unassignToken(serial, user, None)

            c.audit['success'] = res
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            if "" == c.audit['realm']:
                c.audit['realm'] = getDefaultRealm()

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, 1)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def assign(self, action, **params):
        """
        method:
            admin/assign

        description:
            assigns a token to a user, i.e. a binding between the token and
            the user is created.

        arguments:
            * serial     - required - the serial number/identifier of the token
            * user       - required - login user name
            * pin        - optional - the pin of the user pass

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params

        try:

            upin = getParam(param, "pin", optional)
            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'assign', param)

            log.info("assigning token with serial %s to user %s@%s" %
                     (serial, user.login, user.realm))
            res = assignToken(serial, user, upin, param)

            c.audit['success'] = res
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error('token assignment failed! %r' % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, 0)

        finally:
            Session.close()

########################################################
    @log_with(log)
    def setPin(self, action, **params):
        """
        method:
            admin/set

        description:
            This function sets the smartcard PINs of a eTokenNG OTP.
            The userpin is used to store the mOTP PIN of mOTP tokens!
            !!! For setting the OTP PIN, use the function /admin/set!

        arguments:
            * serial     - required
            * userpin    - optional: store the userpin
            * sopin      - optional: store the sopin

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        count = 0

        description = "setPin: parameters are\
        serial\
        userpin\
        sopin\
        "
        try:
            param = getLowerParams(request.params)

            # if there is a pin
            if "userpin" in param:
                msg = "setting userPin failed"
                userPin = getParam(param, "userpin", required)
                serial = getParam(param, "serial", required)

                # check admin authorization
                self.Policy.checkPolicyPre('admin', 'setPin', param)

                log.info("setting userPin for token with serial %s" % serial)
                ret = setPinUser(userPin, serial)
                res["set userpin"] = ret
                count = count + 1
                c.audit['action_detail'] += "userpin, "

            if "sopin" in param:
                msg = "setting soPin failed"
                soPin = getParam(param, "sopin", required)
                serial = getParam(param, "serial", required)

                # check admin authorization
                self.Policy.checkPolicyPre('admin', 'setPin', param)

                log.info("setting soPin for token with serial %s" % serial)
                ret = setPinSo(soPin, serial)
                res["set sopin"] = ret
                count = count + 1
                c.audit['action_detail'] += "sopin, "

            if count == 0:
                Session.rollback()
                return sendError(response,
                                 ParameterError("Usage: %s" % description,
                                                id=77))

            c.audit['success'] = count

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error('policy failed %r, %r' % (msg, pe))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error('%s :%r' % (msg, e))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(e), 0)

        finally:
            Session.close()

########################################################
    @log_with(log)
    def set(self, action, **params):
        """
        method:
            admin/set

        description:
            this function is used to set many different values of a token.

        arguments:
            * serial     - optional
            * user       - optional
            * pin        - optional - set OTP PIN
            * MaxFailCount  - optional - set maximum fail counter of a token
            * SyncWindow    - optional - set synchronization window of a token
            * OtpLen        - optional - set OTP Lenght of the token
            * CounterWindow - optional - set counter window (blank presses)
            * hashlib       - optioanl - set hashing algo for HMAC tokens.
                                         This can be sha1, sha256, sha512
            * timeWindow    - optional - set synchronize window for timebased
                                         tokens (in seconds)
            * timeStep      - optional - set timestep for timebased tokens
                                         (usually 30 or 60 seconds)
            * timeShift     - optional - set shift or timedrift of this token
            * countAuthSuccessMax    - optional    - set maximum allowed
                                                    successful authentications
            * countAuthSuccess\      - optional    - set counter of the
                                                    successful authentications
            * countAuth        - optional - set counter of authentications
            * countAuthMax     - optional - set maximum allowed authentication
                                            tries
            * validityPeriodStart - optional - set start date of the
                                               validity period. The token can
                                               not be used before this date
            * validityPeriodEnd   - optional - set end date of the validaity
                                               period. The token can not be
                                               used after this date
            * phone - set the phone number for an SMS token

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        count = 0

        description = "set: parameters are\
        pin\
        MaxFailCount\
        SyncWindow\
        OtpLen\
        CounterWindow\
        hashlib\
        timeWindow\
        timeStep\
        timeShift\
        countAuthSuccessMax\
        countAuthSuccess\
        countAuth\
        countAuthMax\
        validityPeriodStart\
        validityPeriodEnd\
        description\
        phone\
        "
        msg = ""

        try:
            param = getLowerParams(request.params)

            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'set', param, user=user)

            # if there is a pin
            if "pin" in param:
                msg = "setting pin failed"
                upin = getParam(param, "pin", required)
                log.info("setting pin for token with serial %r" % serial)
                if serial is not None:
                    tokenrealms = getTokenRealms(serial)
                else:
                    tokenrealms = []
                if 1 == self.Policy.getOTPPINEncrypt(serial=serial, user=user,
                                                     tokenrealms=tokenrealms):
                    param['encryptpin'] = "True"
                ret = setPin(upin, user, serial, param)
                res["set pin"] = ret
                count = count + 1
                c.audit['action_detail'] += "pin, "

            if "MaxFailCount".lower() in param:
                msg = "setting MaxFailCount failed"
                maxFail = int(getParam(param, "MaxFailCount".lower(),
                                       required))
                log.info("setting maxFailCount (%r) for token with "
                         "serial %r" % (maxFail, serial))
                ret = setMaxFailCount(maxFail, user, serial)
                res["set MaxFailCount"] = ret
                count = count + 1
                c.audit['action_detail'] += "maxFailCount=%d, " % maxFail

            if "SyncWindow".lower() in param:
                msg = "setting SyncWindow failed"
                syncWindow = int(getParam(param, "SyncWindow".lower(),
                                          required))
                log.info("setting syncWindow (%r) for token with serial %r" %
                         (syncWindow, serial))
                ret = setSyncWindow(syncWindow, user, serial)
                res["set SyncWindow"] = ret
                count = count + 1
                c.audit['action_detail'] += "syncWindow=%d, " % syncWindow

            if "description" in param:
                msg = "setting description failed"
                description = getParam(param, "description".lower(), required)
                log.info("setting description (%r) for token with serial %r" %
                         (description, serial))
                ret = setDescription(description, user, serial)
                res["set description"] = ret
                count = count + 1
                c.audit['action_detail'] += "description=%r, " % description

            if "CounterWindow".lower() in param:
                msg = "setting CounterWindow failed"
                cWindow = int(getParam(param, "CounterWindow".lower(),
                                       required))
                log.info("setting counterWindow (%r) for token with "
                         "serial %r" % (cWindow, serial))
                ret = setCounterWindow(cWindow, user, serial)
                res["set CounterWindow"] = ret
                count = count + 1
                c.audit['action_detail'] += "counterWindow=%d, " % cWindow

            if "OtpLen".lower() in param:
                msg = "setting OtpLen failed"
                otpLen = int(getParam(param, "OtpLen".lower(), required))
                log.info("setting OtpLen (%r) for token with serial %r" %
                         (otpLen, serial))
                ret = setOtpLen(otpLen, user, serial)
                res["set OtpLen"] = ret
                count = count + 1
                c.audit['action_detail'] += "otpLen=%d, " % otpLen

            if "hashlib" in param:
                msg = "setting hashlib failed"
                hashlib = getParam(param, "hashlib".lower(), required)
                log.info("setting hashlib (%r) for token with serial %r" %
                         (hashlib, serial))
                ret = setHashLib(hashlib, user, serial)
                res["set hashlib"] = ret
                count = count + 1
                c.audit['action_detail'] += "hashlib=%s, " % unicode(hashlib)

            if "timeWindow".lower() in param:
                msg = "setting timeWindow failed"
                timeWindow = int(getParam(param, "timeWindow".lower(),
                                          required))
                log.info("setting timeWindow (%r) for token with serial %r" %
                         (timeWindow, serial))
                ret = addTokenInfo("timeWindow", timeWindow, user, serial)
                res["set timeWindow"] = ret
                count = count + 1
                c.audit['action_detail'] += "timeWindow=%d, " % timeWindow

            if "timeStep".lower() in param:
                msg = "setting timeStep failed"
                timeStep = int(getParam(param, "timeStep".lower(), required))
                log.info("setting timeStep (%r) for token with serial %r" %
                         (timeStep, serial))
                ret = addTokenInfo("timeStep", timeStep, user, serial)
                res["set timeStep"] = ret
                count = count + 1
                c.audit['action_detail'] += "timeStep=%d, " % timeStep

            if "timeShift".lower() in param:
                msg = "setting timeShift failed"
                timeShift = int(getParam(param, "timeShift".lower(), required))
                log.info("setting timeShift (%r) for token with serial %r" %
                         (timeShift, serial))
                ret = addTokenInfo("timeShift", timeShift, user, serial)
                res["set timeShift"] = ret
                count = count + 1
                c.audit['action_detail'] += "timeShift=%d, " % timeShift

            if "countAuth".lower() in param:
                msg = "setting countAuth failed"
                ca = int(getParam(param, "countAuth".lower(), required))
                log.info("setting count_auth (%r) for token with serial %r" %
                         (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_count_auth(int(ca))
                    count = count + 1
                    ret += 1
                res["set countAuth"] = ret
                c.audit['action_detail'] += "countAuth=%d, " % ca

            if "countAuthMax".lower() in param:
                msg = "setting countAuthMax failed"
                ca = int(getParam(param, "countAuthMax".lower(), required))
                log.info("setting count_auth_max (%r) for token with "
                         "serial %r" % (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_count_auth_max(int(ca))
                    count = count + 1
                    ret += 1
                res["set countAuthMax"] = ret
                c.audit['action_detail'] += "countAuthMax=%d, " % ca

            if "countAuthSuccess".lower() in param:
                msg = "setting countAuthSuccess failed"
                ca = int(getParam(param, "countAuthSuccess".lower(), required))
                log.info("setting count_auth_success (%r) for token with"
                         " serial %r" % (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_count_auth_success(int(ca))
                    count = count + 1
                    ret += 1
                res["set countAuthSuccess"] = ret
                c.audit['action_detail'] += "countAuthSuccess=%d, " % ca

            if "countAuthSuccessMax".lower() in param:
                msg = "setting countAuthSuccessMax failed"
                ca = int(getParam(param, "countAuthSuccessMax".lower(),
                                  required))
                log.info("setting count_auth_success_max (%r) for token with"
                         " serial %r" % (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_count_auth_success_max(int(ca))
                    count = count + 1
                    ret += 1
                res["set countAuthSuccessMax"] = ret
                c.audit['action_detail'] += "countAuthSuccessMax=%d, " % ca

            if "validityPeriodStart".lower() in param:
                msg = "setting validityPeriodStart failed"
                ca = getParam(param, "validityPeriodStart".lower(), required)
                log.info("setting validity_period_start (%r) for token "
                         "with serial %r" % (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_validity_period_start(ca)
                    count = count + 1
                    ret += 1
                res["set validityPeriodStart"] = ret
                c.audit['action_detail'] += u"validityPeriodStart=%s, " % unicode(ca)

            if "validityPeriodEnd".lower() in param:
                msg = "setting validityPeriodEnd failed"
                ca = getParam(param, "validityPeriodEnd".lower(), required)
                log.info("setting validity_period_end (%r) for token with"
                         " serial %r" % (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.set_validity_period_end(ca)
                    count = count + 1
                    ret += 1
                res["set validityPeriodEnd"] = ret
                c.audit['action_detail'] += "validityPeriodEnd=%s, " % unicode(ca)

            if "phone" in param:
                msg = "setting phone failed"
                ca = getParam(param, "phone".lower(), required)
                log.info("setting phone (%r) for token with serial %r" %
                         (ca, serial))
                tokens = getTokens4UserOrSerial(user, serial)
                ret = 0
                for tok in tokens:
                    tok.addToTokenInfo("phone", ca)
                    count = count + 1
                    ret += 1
                res["set phone"] = ret
                c.audit['action_detail'] += "phone=%s, " % unicode(ca)

            if count == 0:
                Session.rollback()
                return sendError(response,
                                 ParameterError("Usage: %s" % description,
                                                id=77))

            c.audit['success'] = count
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error('policy failed: %s, %r' % (msg, pe))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error('%s :%r' % (msg, e))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()

########################################################
    @log_with(log)
    def resync(self, action, **params):
        """
        method:
            admin/resync - resync a token to a new counter

        description:
            this function resync the token, if the counter on server side
            is out of sync with the physical token.

        arguments:
            * serial     - serial or user required
            * user       - s.o.
            * otp1       - the next otp to be found
            * otp2       - the next otp after the otp1

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params
        try:
            serial = getParam(param, "serial", optional)
            user = getUserFromParam(param, optional)

            otp1 = getParam(param, "otp1", required)
            otp2 = getParam(param, "otp2", required)

            ''' to support the challenge based resync, we have to pass the challenges
                down to the token implementation
            '''
            chall1 = getParam(param, "challenge1", optional)
            chall2 = getParam(param, "challenge2", optional)

            options = None
            if chall1 is not None and chall2 is not None:
                options = {'challenge1': chall1,
                           'challenge2': chall2}

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'resync', param)

            log.info("resyncing token with serial %r, user %r@%r"
                     % (serial, user.login, user.realm))
            res = resyncToken(otp1, otp2, user, serial, options)

            c.audit['success'] = res
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm
            if "" == c.audit['realm'] and "" != c.audit['user']:
                c.audit['realm'] = getDefaultRealm()

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error('resyncing token failed %r' % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, 1)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def userlist(self, action, **params):
        """
        method:
            admin/userlist - list all users

        description:
            lists the user in a realm

        arguments:
            * <searchexpr> - will be retrieved from the UserIdResolverClass
            * realm	 - a realm, which is a collection of resolver configurations
            * resConf	 - a destinct resolver configuration

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        users = []
        param = request.params

        # check admin authorization
        # check if we got a realm or resolver, that is ok!
        try:
            realm = getParam(param, "realm", optional)
            #_resolver = getParam(param, "resConf", optional)

            self.Policy.checkPolicyPre('admin', 'userlist',
                           { 'user': "dummy", 'realm':realm})

            up = 0
            user = getUserFromParam(param, optional)

            log.info("displaying users with param: %s, ", param)

            if (len(user.realm) > 0):
                up = up + 1
            if (len(user.conf) > 0):
                up = up + 1
            # Here we need to list the users, that are only visible in the realm!!
            #  we could also only list the users in the realm, if the admin got
            #  the right "userlist".

            ### list searchfields if no other param
            if len(param) == up:
                usage = {"usage":"list available users matching the given search patterns:"}
                usage["searchfields"] = getSearchFields(user)
                res = usage
            else:
                users = getUserList(remove_session_from_param(param), user)
                res = users

                c.audit['success'] = True
                c.audit['info'] = "realm: %s" % realm

            Session.commit()
            return sendResult(response, res)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("failed %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def tokenrealm(self, action, **params):
        '''
        method:
            admin/tokenrealm - set the realms a token belongs to

        description:
            sets the realms of a token

        arguments:
            * serial    - required -  serialnumber of the token
            * realms    - required -  comma seperated list of realms
        '''
        param = request.params
        try:
            serial = getParam(param, "serial", required)
            realms = getParam(param, "realms", required)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'tokenrealm', param)

            log.info("setting realms for token %s to %s" % (serial, realms))
            realmList = realms.split(',')
            ret = setRealms(serial, realmList)

            c.audit['success'] = ret
            c.audit['info'] = realms

            Session.commit()
            return sendResult(response, ret, 1)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error('error setting realms for token %r' % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, e, 1)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def reset(self, action, **params):
        """
        method:
            admin/reset

        description:
            reset the FailCounter of a Token

        arguments:
            user or serial - to identify the tokens

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        param = request.params

        serial = getParam(param, "serial", optional)
        user = getUserFromParam(param, optional)

        try:

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'reset', param , user=user)

            log.info("resetting the FailCounter for token with serial %s" % serial)
            ret = resetToken(user, serial)

            c.audit['success'] = ret
            c.audit['user'] = user.login
            c.audit['realm'] = user.realm

            # DeleteMe: This code will never run, since getUserFromParam
            # always returns a realm!
            #if "" == c.audit['realm'] and "" != c.audit['user']:
            #    c.audit['realm'] = getDefaultRealm()

            Session.commit()
            return sendResult(response, ret)

        except PolicyException as pe:
            log.error('policy failed %r' % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as exx:    # pragma: no cover
            log.error("Error resetting failcounter %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()



########################################################
    @log_with(log)
    def copyTokenPin(self, action, **params):
        """
        method:
            admin/copyTokenPin

        description:
            copies the token pin from one token to another

        arguments:
            * from - required - serial of token from
            * to   - required - serial of token to

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        ret = 0
        err_string = ""
        param = request.params

        try:
            serial_from = getParam(param, "from", required)
            serial_to = getParam(param, "to", required)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'copytokenpin', param)

            log.info("copying Pin from token %s to token %s" % (serial_from, serial_to))
            ret = copyTokenPin(serial_from, serial_to)

            c.audit['success'] = ret
            c.audit['serial'] = serial_to
            c.audit['action_detail'] = "from %s" % serial_from

            err_string = unicode(ret)
            if -1 == ret:
                err_string = "can not get PIN from source token"
            if -2 == ret:
                err_string = "can not set PIN to destination token"
            if 1 != ret:
                c.audit['action_detail'] += ", " + err_string
                c.audit['success'] = 0

            Session.commit()
            # Success
            if 1 == ret:
                return sendResult(response, True)
            else:
                return sendError(response, "copying token pin failed: %s" % err_string)

        except PolicyException as pe:
            log.error("Error doing losttoken %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("Error copying token pin")
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def copyTokenUser(self, action, **params):
        """
        method:
            admin/copyTokenUser

        description:
            copies the token user from one token to another

        arguments:
            * from - required - serial of token from
            * to   - required - serial of token to

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        ret = 0
        err_string = ""
        param = request.params

        try:

            serial_from = getParam(param, "from", required)
            serial_to = getParam(param, "to", required)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'copytokenuser', param)

            log.info("copying User from token %s to token %s" % (serial_from, serial_to))
            ret = copyTokenUser(serial_from, serial_to)

            c.audit['success'] = ret
            c.audit['serial'] = serial_to
            c.audit['action_detail'] = "from %s" % serial_from

            err_string = unicode(ret)
            if -1 == ret:
                err_string = "can not get user from source token"
            if -2 == ret:
                err_string = "can not set user to destination token"
            if 1 != ret:
                c.audit['action_detail'] += ", " + err_string
                c.audit['success'] = 0

            Session.commit()
            # Success
            if 1 == ret:
                return sendResult(response, True)
            else:
                return sendError(response, "copying token user failed: %s" % err_string)

        except PolicyException as pe:
            log.error("Error doing losttoken %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("Error copying token user")
            Session.rollback()
            return sendError(response, e)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def losttoken(self, action, **params):
        """
        method:
            admin/losttoken

        description:
            creates a new password token and copies the PIN and the
            user of the old token to the new token.
            The old token is disabled.

        arguments:
            * serial - serial of the old token

        returns:
            a json result with the new serial an the password

        exception:
            if an error occurs an exception is serialized and returned

        """
        ret = 0
        res = {}
        param = request.params

        try:

            serial = getParam(param, "serial", required)

            # check admin authorization
            self.Policy.checkPolicyPre('admin', 'losttoken', param)

            res = losttoken(serial)

            c.audit['success'] = ret
            c.audit['serial'] = res.get('serial')
            c.audit['action_detail'] = "from %s" % serial

            Session.commit()
            return sendResult(response, res)

        except PolicyException as pe:
            log.error("Error doing losttoken %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("Error doing losttoken %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(e))

        finally:
            Session.close()


########################################################
    @log_with(log)
    def loadtokens(self, action, **params):
        """
        method:
            admin/loadtokens

        description:
            loads a whole token file to the server

        arguments:
            * file -  the file in a post request
            * type -  the file type.

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = "Loading token file failed!"
        known_types = ['aladdin-xml', 'oathcsv', 'yubikeycsv']
        TOKENS = {}
        res = None

        sendResultMethod = sendResult
        sendErrorMethod = sendError

        try:
            from privacyideaee.lib.ImportOTP import getKnownTypes
            known_types.extend(getKnownTypes())
            log.info("importing privacyideaee.lib. Running Enterprise Edition. Known import types: %s" % known_types)

            from privacyideaee.lib.ImportOTP.PSKC import parsePSKCdata
            log.info("loaded parsePSKCdata")

            from privacyideaee.lib.ImportOTP.DPWplain import parseDPWdata
            log.info("loaded parseDPWdata")

            from privacyideaee.lib.ImportOTP.eTokenDat import parse_dat_data
            log.info("loaded parseDATdata")

            from privacyideaee.lib.ImportOTP.vasco import parseVASCOdata
            log.info("loaded parseVASCOdata")

        except Exception as e:
            log.warning("Unable to load privacyideaee.lib. Running Community Edition: %r" % e)

        try:
            log.debug("getting POST request")
            log.debug(request.POST)
            tokenFile = request.POST['file']
            fileType = request.POST['type']

            pskc_type = None
            pskc_password = None
            pskc_preshared = None
            pskc_checkserial = False

            hashlib = None

            if "pskc" == fileType:
                pskc_type = request.POST['pskc_type']
                pskc_password = request.POST['pskc_password']
                pskc_preshared = request.POST['pskc_preshared']
                if 'pskc_checkserial' in request.POST:
                    pskc_checkserial = True

            fileString = ""
            typeString = ""

            log.debug("loading token file to server using POST request. Filetype: %s. File: %s"
                        % (fileType, tokenFile))

            # In case of form post requests, it is a "instance" of FieldStorage
            # i.e. the Filename is selected in the browser and the data is transferred
            # in an iframe. see: http://jquery.malsup.com/form/#sample4
            #
            if type(tokenFile).__name__ == 'instance':
                log.debug("Field storage file: %s", tokenFile)
                fileString = tokenFile.value
                sendResultMethod = sendXMLResult
                sendErrorMethod = sendXMLError
            else:
                fileString = tokenFile
            log.debug("fileString: %s", fileString)

            if type(fileType).__name__ == 'instance':
                log.debug("Field storage type: %s", fileType)
                typeString = fileType.value
            else:
                typeString = fileType
            log.debug("typeString: <<%s>>", typeString)
            if "pskc" == typeString:
                log.debug("passing password: %s, key: %s, checkserial: %s" % (pskc_password, pskc_preshared, pskc_checkserial))

            if fileString == "" or typeString == "":
                log.error("file: %s", fileString)
                log.error("type: %s", typeString)
                log.error("Error loading/importing token file. file or type empty!")
                return sendErrorMethod(response, "Error loading tokens. File or Type empty!")

            if typeString not in known_types:
                log.error("Unknown file type: >>%s<<. We only know the types: %s" % (typeString, ', '.join(known_types)))
                return sendErrorMethod(response, "Unknown file type: >>%s<<. We only know the types: %s" % (typeString, ', '.join(known_types)))

            # Parse the tokens from file and get dictionary
            if typeString == "aladdin-xml":
                TOKENS = parseSafeNetXML(fileString)
                # we only do hashlib for aladdin at the moment.
                if 'aladdin_hashlib' in request.POST:
                    hashlib = request.POST['aladdin_hashlib']
            elif typeString == "oathcsv":
                TOKENS = parseOATHcsv(fileString)
            elif typeString == "yubikeycsv":
                TOKENS = parseYubicoCSV(fileString)
            elif typeString == "dpw":
                TOKENS = parseDPWdata(fileString)

            elif typeString == "dat":
                startdate = request.POST.get('startdate', None)
                TOKENS = parse_dat_data(fileString, startdate)

            elif typeString == "feitian":
                TOKENS = parsePSKCdata(fileString, do_feitian=True)
            elif typeString == "pskc":
                if "key" == pskc_type:
                    TOKENS = parsePSKCdata(fileString, preshared_key_hex=pskc_preshared, do_checkserial=pskc_checkserial)
                elif "password" == pskc_type:
                    TOKENS = parsePSKCdata(fileString, password=pskc_password, do_checkserial=pskc_checkserial)
                    #log.debug(TOKENS)
                elif "plain" == pskc_type:
                    TOKENS = parsePSKCdata(fileString, do_checkserial=pskc_checkserial)
            elif typeString == "vasco":
                vasco_otplen = request.POST['vasco_otplen']
                (fh, filename) = mkstemp()
                f = open(filename, "w")
                f.write(fileString)
                f.close()
                TOKENS = parseVASCOdata(filename, int(vasco_otplen))
                os.remove(filename)
                if TOKENS is None:
                    raise ImportException("Vasco DLL was not properly loaded. Importing of VASCO token not possible. Please check the log file for more details.")


            log.debug("read %i tokens. starting import now" % len(TOKENS))

            # Now import the Tokens from the dictionary
            ret = ""
            for serial in TOKENS:
                log.debug("importing token %s" % TOKENS[serial])

                # this needs to return the valid realms of the admin.
                # it also checks the license token number
                res = self.Policy.checkPolicyPre('admin', 'import', {})
                # we put the token in the FIRST realm of the admin.
                # so tokenrealm will either be ONE realm or NONE
                log.info("setting tokenrealm %s" % res['realms'])
                tokenrealm = None
                if res['realms']:
                    tokenrealm = res.get('realms')[0]


                log.info("initialize token. serial: %s, realm: %s" % (serial, tokenrealm))

                ## for the eToken dat we assume, that it brings all its
                ## init parameters in correct format
                if typeString == "dat":
                    init_param = TOKENS[serial]

                else:
                    init_param = {'serial':serial,
                                    'type':TOKENS[serial]['type'],
                                    'description': TOKENS[serial].get("description", "imported"),
                                    'otpkey' : TOKENS[serial]['hmac_key'],
                                    'otplen' : TOKENS[serial].get('otplen'),
                                    'timeStep' : TOKENS[serial].get('timeStep'),
                                    'hashlib' : TOKENS[serial].get('hashlib')}



                # add additional parameter for vasco tokens
                if TOKENS[serial]['type'] == "vasco":
                    init_param['vasco_appl'] = TOKENS[serial]['tokeninfo'].get('application')
                    init_param['vasco_type'] = TOKENS[serial]['tokeninfo'].get('type')
                    init_param['vasco_auth'] = TOKENS[serial]['tokeninfo'].get('auth')

                # add ocrasuite for ocra tokens, only if ocrasuite is not empty
                if TOKENS[serial]['type'] == 'ocra':
                    if TOKENS[serial].get('ocrasuite', "") != "":
                        init_param['ocrasuite'] = TOKENS[serial].get('ocrasuite')

                if hashlib and hashlib != "auto":
                    init_param['hashlib'] = hashlib

                if tokenrealm:
                    self.Policy.checkPolicyPre('admin', 'loadtokens',
                                   {'tokenrealm': tokenrealm })

                (ret, tokenObj) = initToken(init_param, User('', '', ''),
                                            tokenrealm=tokenrealm)

            log.info ("%i tokens imported." % len(TOKENS))
            res = { 'value' : True, 'imported' : len(TOKENS) }

            c.audit['info'] = "%s, %s (imported: %i)" % (fileType, tokenFile, len(TOKENS))
            c.audit['serial'] = ', '.join(TOKENS.keys())
            logTokenNum()
            c.audit['success'] = ret

            Session.commit()
            return sendResultMethod(response, res)

        except PolicyException as pe:
            log.error("Failed checking policy: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe), 1)

        except Exception as e:    # pragma: no cover
            log.error("failed! %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendErrorMethod(response, unicode(e))

        finally:
            Session.close()


    @log_with(log)
    def testresolver(self, action, **params):
        """
        method:
            admin/testresolver

        description:
            This method tests a useridresolvers configuration

        arguments:
            * type     - "LDAP": depending on the type there are other parameters:
                       - "SQL"

            * LDAP:
                * BINDDN
                * BINDPW
                * LDAPURI
                * TIMEOUT
                * LDAPBASE
                * LOGINNAMEATTRIBUTE
                * LDAPSEARCHFILTER
                * LDAPFILTER
                * USERINFO
                * LDAPSEARCHFILTER
                * SIZELIMIT
                * NOREFERRALS
                * CACERTIFICATE

            * SQL:
                * Driver
                * Server
                * Port
                * Database
                * User
                * Password
                * Table
                
            * SCIM:
                * Authserver
                * Resourceserver
                * Client
                * Secret
                * Map

        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}

        try:
            param = getLowerParams(request.params)
            typ = getParam(param, "type", required)

            if typ == "ldap":
                import privacyidea.lib.resolvers.LDAPIdResolver

                param['BINDDN'] = getParam(param, "ldap_binddn", required)
                param['BINDPW'] = getParam(param, "ldap_password", required)
                param['LDAPURI'] = getParam(param, "ldap_uri", required)
                param['TIMEOUT'] = getParam(param, "ldap_timeout", required)
                param['LDAPBASE'] = getParam(param, "ldap_basedn", required)
                param['LOGINNAMEATTRIBUTE'] = getParam(param, "ldap_loginattr", required)
                param['LDAPSEARCHFILTER'] = getParam(param, "ldap_searchfilter", required)
                param['LDAPFILTER'] = getParam(param, "ldap_userfilter", required)
                param['USERINFO'] = getParam(param, "ldap_mapping", required)
                param['SIZELIMIT'] = getParam(param, "ldap_sizelimit", required)
                param['NOREFERRALS'] = getParam(param, "noreferrals", optional)
                param['CACERTIFICATE'] = getParam(param, "ldap_certificate", optional)

                (success, desc) = privacyidea.lib.resolvers.LDAPIdResolver.IdResolver.testconnection(param)
                res['result'] = success
                res['desc'] = desc

            elif typ == "sql":
                import privacyidea.lib.resolvers.SQLIdResolver

                param["Driver"] = getParam(param, "sql_driver", required)
                param["Server"] = getParam(param, "sql_server", required)
                param["Port"] = getParam(param, "sql_port", required)
                param["Database"] = getParam(param, "sql_database", required)
                param["User"] = getParam(param, "sql_user", required)
                param["Password"] = getParam(param, "sql_password", required)
                param["Table"] = getParam(param, "sql_table", required)
                param["Where"] = getParam(param, "sql_where", optional)

                (num, err_str) = privacyidea.lib.resolvers.SQLIdResolver.IdResolver.testconnection(param)
                res['result'] = (num >= 0)
                res['rows'] = num
                res['err_string'] = err_str

            elif typ == "scim":
                import privacyidea.lib.resolvers.SCIMIdResolver
                param["Authserver"] = getParam(param, "authserver", required)
                param["Resourceserver"] = getParam(param, "resourceserver", required)
                param["Client"] = getParam(param, "client", required)
                param["Secret"] = getParam(param, "secret", required)
                param["Map"] = getParam(param, "map", required)

                (num, err_str) = privacyidea.lib.resolvers.SCIMIdResolver.IdResolver.testconnection(param)
                res['result'] = (num >= 0)
                res['rows'] = num
                res['err_string'] = err_str

                 

            Session.commit()
            return sendResult(response, res)

        except Exception as e:    # pragma: no cover
            log.error("failed: %r" % e)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(e), 1)

        finally:
            Session.close()


    @log_with(log)
    def checkstatus(self, action, **params):
        """
        show the status either

        * of one dedicated challenge
        * of all challenges of a token
        * of all challenges belonging to all tokens of a user

        :param transactionid/state:  the transaction id of the challenge
        :param serial: serial number of the token - will show all challenges
        :param user:

        :return: json result of token and challenges

        """
        res = {}
        param = {}


        description = """
            admin/checkstatus: check the token status -
            for assynchronous verification. Missing parameter:
            You need to provide one of the parameters "transactionid", "user" or "serial"'
            """

        try:

            param.update(request.params)
            self.Policy.checkPolicyPre('admin', "checkstatus")

            transid = param.get('transactionid', None) or param.get('state', None)
            user = getUserFromParam(param, optional)
            serial = getParam(param, 'serial'          , optional)

            if transid is None and user.isEmpty() and serial is None:
                ## raise exception
                log.error("missing parameter: "
                             "transactionid, user or serial number for token")
                raise ParameterError("Usage: %s" % description, id=77)

            ## gather all challenges from serial, transactionid and user
            challenges = set()
            if serial is not None:
                challenges.update(get_challenges(serial=serial))

            if transid is not None :
                challenges.update(get_challenges(transid=transid))

            ## if we have a user
            if user.isEmpty() == False:
                tokens = getTokens4UserOrSerial(user=user)
                for token in tokens:
                    serial = token.getSerial()
                    challenges.update(get_challenges(serial=serial))

            serials = set()
            for challenge in challenges:
                serials.add(challenge.getTokenSerial())

            status = {}
            ## sort all information by token serial number
            for serial in serials:
                stat = {}
                chall_dict = {}

                ## add the challenges info to the challenge dict
                for challenge in challenges:
                    if challenge.getTokenSerial() == serial:
                        chall_dict[challenge.getTransactionId()] = challenge.get_vars(save=True)
                stat['challenges'] = chall_dict

                ## add the token info to the stat dict
                tokens = getTokens4UserOrSerial(serial=serial)
                token = tokens[0]
                stat['tokeninfo'] = token.get_vars(save=True)

                ## add the local stat to the summary status dict
                status[serial] = stat

            res['values'] = status
            c.audit['success'] = res

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pe:
            log.error("policy failed: %r" % pe)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(pe))

        except Exception as exx:    # pragma: no cover
            log.error("failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendResult(response, unicode(exx), 0)

        finally:
            Session.close()


#eof###########################################################################




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  Pylons application test package

This package assumes the Pylons environment is already loaded, such as
when this script is imported from the `nosetests --with-pylons=test.ini`
command.

This module initializes the application via ``websetup`` (`paster
setup-app`) and provides the base testing objects.

  Dependencies: -

'''
import json

import pylons.test
import os
import logging

from unittest import TestCase

from paste.deploy import appconfig
from paste.deploy import loadapp
from paste.script.appinstall import SetupCommand

from pylons import config, url
from pylons.configuration import config as env
from routes.util import URLGenerator
from webtest import TestApp

from privacyidea.config.environment import load_environment

import warnings
warnings.filterwarnings(action='ignore', category=DeprecationWarning)


def fxn():
    warnings.warn("deprecated", DeprecationWarning)

with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    fxn()

from privacyidea.websetup import setup_app

LOG = logging.getLogger(__name__)

__all__ = ['environ', 'url', 'TestController']
SetupCommand('setup-app').run([config['__file__']])

environ = {}


def setUpPackage():
    '''
    setUpPackage is called before each test package / class

    this hook is used to re-initialize the database
    '''
    SetupCommand('setup-app').run([config['__file__']])
    return


def tearDownPackage():
    '''
    tearDownPackage is called when a test package is finished
    '''
    return


class TestController(TestCase):
    '''
    the TestController, which loads the privacyidea app upfront
    '''
    def __init__(self, *args, **kwargs):
        '''
        initialize the test class
        '''
        TestCase.__init__(self, *args, **kwargs)

        LOG.error("ConfigFile: %s " % config['__file__'])

        conffile = config['__file__']

        if pylons.test.pylonsapp:
            wsgiapp = pylons.test.pylonsapp
        else:
            wsgiapp = loadapp('config: %s' % config['__file__'])

        self.app = TestApp(wsgiapp)

        conf = None
        if conffile.startswith('/'):
            conf = appconfig('config:%s' % config['__file__'],
                             relative_to=None)
        else:
            raise Exception('dont know how to load the application relatively')
        # conf = appconfig('config: %s' % config['__file__'], relative_to=rel)

        load_environment(conf.global_conf, conf.local_conf)
        self.appconf = conf

        url._push_object(URLGenerator(config['routes.map'], environ))

        self.isSelfTest = False
        if "privacyidea.selfTest" in env:
            self.isSelfTest = True

        self.license = 'CE'
                      
        return

    @classmethod
    def unassign(self, serial):
        if pylons.test.pylonsapp:
            wsgiapp = pylons.test.pylonsapp
        else:
            wsgiapp = loadapp('config: %s' % config['__file__'])
        app = TestApp(wsgiapp)
        app.get(url(controller='admin', action='unassign'),
                {'serial': serial})

    @classmethod
    def setupClass(cls):
        '''setup - create clean execution context by resetting database '''
        LOG.info("######## setup_class: %r" % cls)
        SetupCommand('setup-app').run([config['__file__']])
        cls.setup_realms()
        return

    @classmethod
    def tearDownClass(cls):
        '''teardown - cleanup of test class execution result'''
        LOG.info("######## teardown_class: %r" % cls)
        cls.cleanup()
        return

    def setUp(self):
        ''' here we do the system test init per test method '''
        # self.__createResolvers__()
        # self.__createRealms__()
        return

    def tearDown(self):
        # self.__deleteAllRealms__()
        # self.__deleteAllResolvers__()
        return

    @classmethod
    def setup_realms(cls):
        if pylons.test.pylonsapp:
            wsgiapp = pylons.test.pylonsapp
        else:
            wsgiapp = loadapp('config: %s' % config['__file__'])
        app = TestApp(wsgiapp)
        parameters = {'name': 'myDefRes',
                      'fileName': '%(here)s/tests/testdata/def-passwd',
                      'type': 'passwdresolver'
                      }
        resp = app.get(url(controller='system',
                           action='setResolver'),
                       params=parameters)
        assert('"value": true' in resp)

        parameters = {'name': 'myOtherRes',
                      'fileName': '%(here)s/tests/testdata/myDom-passwd',
                      'type': 'passwdresolver'
                      }
        resp = app.get(url(controller='system',
                           action='setResolver'),
                       params=parameters)
        assert('"value": true' in resp)
        # create realms
        parameters = {'realm': 'myDefRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.myDefRes'
                      }
        resp = app.get(url(controller='system',
                           action='setRealm'),
                       params=parameters)
        assert('"value": true' in resp)

        parameters = {'realm': 'myOtherRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.myOtherRes'
                      }
        resp = app.get(url(controller='system',
                           action='setRealm'),
                       params=parameters)
        assert('"value": true' in resp)

        parameters = {'realm': 'myMixRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.'
                      'myOtherRes,privacyidea.lib.resolvers.PasswdIdResolver.'
                      'IdResolver.myDefRes'
                      }
        resp = app.get(url(controller='system',
                           action='setRealm'),
                       params=parameters)
        assert('"value": true' in resp)
        
        # create resolvers
        response = app.get(url(controller='system', action='setResolver'),
                           params={'name': 'reso1',
                                   'type': 'passwdresolver',
                                   'fileName': '%(here)s/tests/testdata/'
                                   'my-passwd'})
        print response
        assert '"value": true' in response

        response = app.get(url(controller='system', action='setResolver'),
                           params={'name': 'reso2',
                                   'type': 'passwdresolver',
                                   'fileName': '%(here)s/tests/testdata/'
                                   'my-pass2'})
        print response
        assert '"value": true' in response
        
        response = app.get(url(controller='system', action='getResolvers'),
                           params={})
        print response
        assert '"status": true' in response

        # create realms

        response = app.get(url(controller='system', action='setRealm'),
                           params={'realm': 'realm1',
                                   'resolvers': 'privacyidea.lib.resolvers.'
                                   'PasswdIdResolver.IdResolver.reso1'})
        print response
        assert '"value": true' in response

        response = app.get(url(controller='system', action='setRealm'),
                           params={'realm': 'realm2',
                                   'resolvers': 'privacyidea.lib.resolvers.'
                                   'PasswdIdResolver.IdResolver.reso2'})
        print response
        assert '"value": true' in response

        # create token
        response = app.get(url(controller='admin', action='init'),
                           params={'serial': 'token1',
                                   'type': 'spass',
                                   'pin': 'secret',
                                   'user': 'heinz',
                                   'realm': 'realm1'
                                   })
        print response
        assert '"value": true' in response

        response = app.get(url(controller='admin', action='init'),
                           params={'serial': 'token2',
                                   'type': 'spass',
                                   'pin': 'secret',
                                   'user': 'nick',
                                   'realm': 'realm1'
                                   })
        print response
        assert '"value": true' in response

        response = app.get(url(controller='admin', action='init'),
                           params={'serial': 'token3',
                                   'type': 'spass',
                                   'pin': 'secret',
                                   'user': 'renate',
                                   'realm': 'realm2'
                                   })
        print response
        assert '"value": true' in response
        
    @classmethod
    def cleanup(cls):
        if pylons.test.pylonsapp:
            wsgiapp = pylons.test.pylonsapp
        else:
            wsgiapp = loadapp('config: %s' % config['__file__'])
        app = TestApp(wsgiapp)
        
        ''' get al realms and delete them '''
        response = app.get(url(controller='system', action='getRealms'))
        jresponse = json.loads(response.body)
        result = jresponse.get("result")
        values = result.get("value", {})
        for realmId in values:
            realm_desc = values.get(realmId)
            realm_name = realm_desc.get("realmname")
            parameters = {"realm": realm_name}
            resp = app.get(url(controller='system', action='delRealm'),
                           params=parameters)
            assert('"result": true' in resp)

        ''' get all resolvers and delete them '''
        response = app.get(url(controller='system',
                               action='getResolvers'))
        jresponse = json.loads(response.body)
        result = jresponse.get("result")
        values = result.get("value", {})
        for realmId in values:
            resolv_desc = values.get(realmId)
            resolv_name = resolv_desc.get("resolvername")
            parameters = {"resolver": resolv_name}
            resp = app.get(url(controller='system', action='delResolver'),
                           params=parameters)
            assert('"status": true' in resp)

    def __deleteAllRealms__(self):
        ''' get al realms and delete them '''
        response = self.app.get(url(controller='system', action='getRealms'))
        jresponse = json.loads(response.body)
        result = jresponse.get("result")
        values = result.get("value", {})
        for realmId in values:
            realm_desc = values.get(realmId)
            realm_name = realm_desc.get("realmname")
            parameters = {"realm": realm_name}
            resp = self.app.get(url(controller='system', action='delRealm'),
                                params=parameters)
            assert('"result": true' in resp)

    def __deleteAllResolvers__(self):
        ''' get all resolvers and delete them '''

        response = self.app.get(url(controller='system',
                                    action='getResolvers'))
        jresponse = json.loads(response.body)
        result = jresponse.get("result")
        values = result.get("value", {})
        for realmId in values:
            resolv_desc = values.get(realmId)
            resolv_name = resolv_desc.get("resolvername")
            parameters = {"resolver": resolv_name}
            resp = self.app.get(url(controller='system', action='delResolver'),
                                params=parameters)
            assert('"status": true' in resp)

    def deleteAllPolicies(self):
        '''
        '''
        response = self.app.get(url(controller='system', action='getPolicy'),)
        self.assertTrue('"status": true' in response, response)

        body = json.loads(response.body)
        policies = body.get('result', {}).get('value', {}).keys()

        for policy in policies:
            self.delPolicy(policy)

        return

    def delPolicy(self, name='otpPin', remoteurl=None):

        parameters = {'name': name,
                      'selftest_admin': 'superadmin'
                      }
        r_url = url(controller='system', action='delPolicy')

        if remoteurl is not None:
            r_url = "%s/%s" % (remoteurl, "system/delPolicy")
            response = do_http(r_url, params=parameters)
        else:
            response = self.app.get(r_url, params=parameters)

        return response

    def deleteAllTokens(self):
        ''' get all tokens and delete them '''

        serials = []

        response = self.app.get(url(controller='admin', action='show'),
                                )
        self.assertTrue('"status": true' in response, response)

        body = json.loads(response.body)
        tokens = body.get('result', {}).get('value', {}).get('data', {})
        for token in tokens:
            serial = token.get("privacyIDEA.TokenSerialnumber")
            serials.append(serial)

        for serial in serials:
            self.removeTokenBySerial(serial)

        return

    def removeTokenBySerial(self, serial):
        ''' delete a token by its serial number '''

        parameters = {"serial": serial}

        response = self.app.get(url(controller='admin', action='remove'),
                                params=parameters)
        return response

    def __createResolvers__(self):
        '''
        create all base test resolvers
        '''
        parameters = {'name': 'myDefRes',
                      'fileName': '%(here)s/tests/testdata/def-passwd',
                      'type': 'passwdresolver'
                      }
        resp = self.app.get(url(controller='system',
                                action='setResolver'),
                            params=parameters)
        assert('"value": true' in resp)

        parameters = {'name': 'myOtherRes',
                      'fileName': '%(here)s/tests/testdata/myDom-passwd',
                      'type': 'passwdresolver'
                      }
        resp = self.app.get(url(controller='system',
                                action='setResolver'),
                            params=parameters)
        assert('"value": true' in resp)

    def __createRealms__(self):
        '''
            Idea: build out of two resolvers
                3 realms
                - 1 per resolver
                - 1 which contains both
            Question:
                search in the mix for the user root must find 2 users
        '''

        parameters = {'realm': 'myDefRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.myDefRes'
                      }
        resp = self.app.get(url(controller='system',
                                action='setRealm'),
                            params=parameters)
        assert('"value": true' in resp)

        parameters = {'realm': 'myOtherRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.myOtherRes'
                      }
        resp = self.app.get(url(controller='system',
                                action='setRealm'),
                            params=parameters)
        assert('"value": true' in resp)

        parameters = {'realm': 'myMixRealm',
                      'resolvers': 'privacyidea.lib.resolvers.'
                      'PasswdIdResolver.IdResolver.'
                      'myOtherRes,privacyidea.lib.resolvers.PasswdIdResolver.'
                      'IdResolver.myDefRes'
                      }
        resp = self.app.get(url(controller='system',
                                action='setRealm'),
                            params=parameters)
        assert('"value": true' in resp)

# ##eof######################################################################



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.859687
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/ocra2token.mako'
_template_uri = '/ocra2token.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("OCRA2 settings")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            # SOURCE LINE 10
            __M_writer(u'\n<table>\n\t<tr><td><label for=ocra2_max_challenge>')
            # SOURCE LINE 12
            __M_writer(escape(_("maximum concurrent OCRA2 challenges")))
            __M_writer(u'</label></td>\n\t\t<td><input type="text" id="ocra2_max_challenge" maxlength="4" class=integer\n\t\t\ttitle=\'')
            # SOURCE LINE 14
            __M_writer(escape(_("This is the maximum concurrent challenges per OCRA2 Token.")))
            __M_writer(u"'/></td></tr>\n\t<tr><td><label for=ocra2_challenge_timeout>")
            # SOURCE LINE 15
            __M_writer(escape(_("OCRA2 challenge timeout")))
            __M_writer(u'</label></td>\n\t\t<td><input type="text" id="ocra2_challenge_timeout" maxlength="6"\n\t\t\ttitle=\'')
            # SOURCE LINE 17
            __M_writer(escape(_("After this time a challenge can not be used anymore. Valid entries are like 1D, 2H or 5M where D=day, H=hour, M=minute.")))
            __M_writer(u"'></td></tr>\n</table>\n\n")
        # SOURCE LINE 21
        __M_writer(u'\n\n')
        # SOURCE LINE 23
        if c.scope == 'enroll.title' :
            # SOURCE LINE 24
            __M_writer(escape(_("OCRA2 - challenge/response Token")))
            __M_writer(u'\n')
        # SOURCE LINE 26
        __M_writer(u'\n')
        # SOURCE LINE 27
        if c.scope == 'enroll' :
            # SOURCE LINE 28
            __M_writer(u"<script>\n\n/*\n * 'typ'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction ocra2_get_enroll_params(){\n    var url = {};\n    url['type'] = 'ocra2';\n   \turl['description'] = $('#enroll_ocra2_desc').val();\n   \turl['sharedsecret'] = 1;\n\turl['ocrasuite'] = $('#ocrasuite_algorithm').val();\n\n    // If we got to generate the ocra2 key, we do it here:\n    if  ( $('#ocra2_key_cb').attr('checked') ) {\n    \turl['genkey'] = 1;\n\n    } else {\n        // OTP Key\n        url['otpkey'] = $('#ocra2_key').val();\n    }\n\n    jQuery.extend(url, add_user_data());\n\n    return url;\n}\n</script>\n<p><span id='ocra2_key_intro'>\n\t")
            # SOURCE LINE 60
            __M_writer(escape(_("Please enter or copy the OCRA2 key.")))
            __M_writer(u'</span></p>\n<table>\n<tr>\n     <td><label for="ocra2_key" id=\'ocra2_key_label\'>')
            # SOURCE LINE 63
            __M_writer(escape(_("OCRA2 key")))
            __M_writer(u'</label></td>\n     <td><input type="text" name="ocra2_key" id="ocra2_key" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr>\n<tr>\n\t<td> </td>\n\t<td><input type=\'checkbox\' id=\'ocra2_key_cb\' onclick="cb_changed(\'ocra2_key_cb\',[\'ocra2_key\',\'ocra2_key_label\',\'ocra2_key_intro\']);">\n\t    <label for=ocra2_key_cb>')
            # SOURCE LINE 69
            __M_writer(escape(_("Generate OCRA2 key.")))
            __M_writer(u'</label></td>\n</tr>\n<tr>\n\t<td><label for="ocrasuite_algorithm">')
            # SOURCE LINE 72
            __M_writer(escape(_("Ocra suite")))
            __M_writer(u'</label></td>\n\t<td><select name="algorithm" id=\'ocrasuite_algorithm\' >\n\t        <option selected value="OCRA-1:HOTP-SHA256-8:C-QN08">SHA256 - otplen 8 digits - numeric challenge 8 digits</option>\n\t        <option value="OCRA-1:HOTP-SHA256-8:C-QN08">SHA256 - otplen 8 digits - numeric challenge 8 digits</option>\n\t        <option value="OCRA-1:HOTP-SHA256-8:C-QA64">SHA256 - otplen 8 digits - numeric challenge 64 chars</option>\n    </select></td>\n</tr>\n<tr>\n    <td><label for="enroll_ocra2_desc" id=\'enroll_ocra2_desc_label\'>')
            # SOURCE LINE 80
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_ocra2_desc" id="enroll_ocra2_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n\n</table>\n\n')
        # SOURCE LINE 87
        __M_writer(u'\n\n\n\n')
        # SOURCE LINE 91
        if c.scope == 'selfservice.title.enroll':
            # SOURCE LINE 92
            __M_writer(escape(_("Enroll your OCRA2 Token")))
            __M_writer(u'\n')
        # SOURCE LINE 94
        __M_writer(u'\n\n')
        # SOURCE LINE 96
        if c.scope == 'selfservice.enroll':
            # SOURCE LINE 97
            __M_writer(u'<script>\n\tjQuery.extend(jQuery.validator.messages, {\n\t\trequired: "')
            # SOURCE LINE 99
            __M_writer(escape(_('required input field')))
            __M_writer(u'",\n\t\tminlength: "')
            # SOURCE LINE 100
            __M_writer(escape(_('minimum length must be greater than {0}')))
            __M_writer(u'",\n\t\tmaxlength: "')
            # SOURCE LINE 101
            __M_writer(escape(_('maximum length must be lower than {0}')))
            __M_writer(u'",\n\n\t});\n\njQuery.validator.addMethod("ocra2_secret", function(value, element, param){\n\tvar res1 = value.match(/^[a-fA-F0-9]+$/i);\n\tvar res2 = !value;\n    return  res1 || res2 ;\n}, \'')
            # SOURCE LINE 109
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u"'  );\n\n$('#form_enroll_ocra2').validate({\n\tdebug: true,\n    rules: {\n        ocra2_secret: {\n            minlength: 40,\n            maxlength: 64,\n            number: false,\n            ocra2_secret: true,\n\t\t\trequired: function() {\n            \tvar res = $('#ocra2_key_cb2').attr('checked') === 'undefined';\n            return res;\n        }\n        }\n    }\n});\n\nfunction self_ocra2_get_param()\n{\n\tvar urlparam = {};\n\tvar typ = 'ocra2';\n\n    if  ( $('#ocra2_key_cb2').attr('checked') ) {\n    \turlparam['genkey'] = 1;\n    } else {\n        // OTP Key\n        urlparam['otpkey'] = $('#ocra2_secret').val();\n    }\n\n\turlparam['type'] \t= typ;\n\turlparam['description'] = $('#ocra2_desc').val();\n\turlparam['sharedsecret'] = '1';\n\treturn urlparam;\n}\n\nfunction self_ocra2_clear()\n{\n\t$('#ocra2_secret').val('');\n\n}\nfunction self_ocra2_submit(){\n\n\tvar ret = false;\n\tvar params =  self_ocra2_get_param();\n\n\tif  ( $('#ocra2_key_cb2').attr('checked') === undefined && $('#form_enroll_ocra2').valid() === false) {\n\t\talert('")
            # SOURCE LINE 156
            __M_writer(escape(_("Form data not valid.")))
            __M_writer(u'\');\n\t} else {\n\t\tenroll_token( params );\n\t\t$("#ocra2_key_cb2").prop("checked", false);\n\t\tcb_changed(\'ocra2_key_cb2\',[\'ocra2_secret\',\'ocra2_key_label2\']);\n\t\tret = true;\n\t}\n\treturn ret;\n\n};\n\nfunction self_ocra2_enroll_details(data) {\n\treturn;\n};\n\n</script>\n<h1>')
            # SOURCE LINE 172
            __M_writer(escape(_("Enroll your OCRA2 Token")))
            __M_writer(u'</h1>\n<div id=\'enroll_ocra2_form\'>\n\t<form class="cmxform" id=\'form_enroll_ocra2\'>\n\t<fieldset>\n\t\t<table><tr>\n\t\t\t<td><label id=\'ocra2_key_label2\' for=\'ocra2_secret\'>')
            # SOURCE LINE 177
            __M_writer(escape(_("Enter seed for the new OCRA2 token:")))
            __M_writer(u'</label></td>\n\t\t\t<td><input id=\'ocra2_secret\' name=\'ocra2_secret\' class="required ui-widget-content ui-corner-all" min="40" maxlength=\'64\'/></td>\n\t\t</tr><tr>\n\t\t\t<td><label for=\'ocra2_key_cb\'>')
            # SOURCE LINE 180
            __M_writer(escape(_("Generate OCRA2 seed")+':'))
            __M_writer(u'</label></td>\n\t\t\t<td><input type=\'checkbox\' name=\'ocra2_key_cb2\' id=\'ocra2_key_cb2\' onclick="cb_changed(\'ocra2_key_cb2\',[\'ocra2_secret\',\'ocra2_key_label2\']);"></td>\n\t\t</tr><tr>\n\t\t\t<td><label id=\'ocra2_desc_label2\' for=\'ocra2_desc\'>')
            # SOURCE LINE 183
            __M_writer(escape(_("Token description")))
            __M_writer(u'</label></td>\n\t\t\t<td><input id=\'ocra2_desc\' name=\'ocra2_desc\' class="ui-widget-content ui-corner-all" value=\'self enrolled\'/></td>\n\t\t</tr>\n        </table>\n\t    <button class=\'action-button\' id=\'button_enroll_ocra2\'\n\t    \t    onclick="self_ocra2_submit();">')
            # SOURCE LINE 188
            __M_writer(escape(_("enroll ocra2 token")))
            __M_writer(u'</button>\n    </fieldset>\n    </form>\n</div>\n\n')
        # SOURCE LINE 194
        __M_writer(u'<!-- -->\n\n')
        # SOURCE LINE 196
        if c.scope == 'selfservice.title.activate':
            # SOURCE LINE 197
            __M_writer(escape(_("Activate your OCRA2 Token")))
            __M_writer(u'\n')
        # SOURCE LINE 199
        __M_writer(u'\n\n')
        # SOURCE LINE 201
        if c.scope == 'selfservice.activate':
            # SOURCE LINE 202
            __M_writer(u'<h1>')
            __M_writer(escape(_("Activate your OCRA2 Token")))
            __M_writer(u'</h1>\n\n<div id=\'oathtokenform2\'>\n\t<form class="cmxform" name=\'myForm\'>\n\t\t<table>\n\t\t<p id=oath_info>\n\t\t<tr><td>')
            # SOURCE LINE 208
            __M_writer(escape(_("Your OCRA2 Token :")))
            __M_writer(u'      </td>\n\t\t    <td> <input type=\'text\' class=\'selectedToken\' class="text ui-widget-content ui-corner-all" disabled\n\t\t    \tvalue=\'\' id=\'serial2\' onchange="resetOcraForm()"/></td></tr>\n\t\t<tr><td><label for=activationcode2>')
            # SOURCE LINE 211
            __M_writer(escape(_("1. Enter the activation code :")))
            __M_writer(u'</label> </td>\n\t\t    <td><input type=\'text\' class="text ui-widget-content ui-corner-all" value=\'\' id=\'activationcode2\'/></td>\n\t\t        <input type=\'hidden\' value=\'')
            # SOURCE LINE 213
            __M_writer(escape(_("Failed to enroll token!")))
            __M_writer(u'\' id=\'ocra2_activate_fail\'/>\n\t\t    <td><div id=\'qr2_activate\'>\n\t\t\t    <button class=\'action-button\' id=\'button_provisionOcra2\' onclick="provisionOcra2(); return false;">\n\t\t\t\t')
            # SOURCE LINE 216
            __M_writer(escape(_("activate your OCRA2 Token")))
            __M_writer(u'\n\t\t\t\t</button>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t\t</tr>\n\t\t<tr><td><div id=\'ocra2_qr_code\'></div></td></tr>\n\t\t</table>\n\t</form>\n\t<form class="cmxform" name=\'myForm2\'>\n\t\t<table>\n\t\t<tr><td><div id=\'qr2_confirm1\'><label for=ocra2_check>')
            # SOURCE LINE 226
            __M_writer(escape(_("2. Enter your confirmation code:")))
            __M_writer(u'\n\t\t\t\t</label></div> </td>\n\t\t    <td><div id=\'qr2_confirm2\'>\n\t\t        <input type=\'hidden\' class="text ui-widget-content ui-corner-all" id=\'transactionid2\' value=\'\' />\n\t\t        <input type=\'hidden\' value=\'')
            # SOURCE LINE 230
            __M_writer(escape(_("OCRA rollout for token %s completed!")))
            __M_writer(u"' \t\t\tid='ocra2_finish_ok'  />\n\t\t        <input type='hidden' value='")
            # SOURCE LINE 231
            __M_writer(escape(_("OCRA token rollout failed! Please retry")))
            __M_writer(u'\' \t\tid=\'ocra2_finish_fail\'/>\n\t\t    \t<input type=\'text\' class="text ui-widget-content ui-corner-all"              \t\tid=\'ocra2_check\' value=\'\' />\n\t\t    \t</div>\n\t\t    </td>\n\t\t\t<td>\n\t\t\t\t<div id=\'qr2_finish\' >\n\t\t\t    <button class=\'action-button\' id=\'button_finishOcra2\' onclick="finishOcra2(); return false;">\n\t\t\t\t')
            # SOURCE LINE 238
            __M_writer(escape(_("finish your OCRA2 Token")))
            __M_writer(u'\n\t\t\t\t</button>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t\t</tr>\n\t\t</div>\n\t\t<tr><td><div id=\'qr2_completed\'></div></td></tr>\n\t\t</table>\n\t</form>\n</div>\n\n\n<script>\n\n\nfunction provisionOcra2() {\n\tshow_waiting();\n\tvar acode = $(\'#activationcode2\').val();\n\tvar serial = $(\'#serial2\').val();\n\tvar activation_fail = $(\'#ocra2_activate_fail\').val();\n\tvar genkey = 1;\n\n\tvar params = {\n\t\t\'type\' : \'ocra2\',\n\t\t\'serial\' : serial,\n\t\t\'genkey\' : 1,\n\t\t\'activationcode\' : acode,\n\t\t\'session\' : get_selfservice_session()\n\t};\n\n\t$.post("/selfservice/useractivateocratoken", params, function(data, textStatus, XMLHttpRequest) {\n\t\thide_waiting();\n\n\t\tif (data.result.status == true) {\n\t\t\tif (data.result.value.activate == true) {\n\t\t\t\t// The token was successfully initialized and we will display the url\n\t\t\t\tshowTokenlist();\n\t\t\t\t// console_log(data.result.value)\n\t\t\t\tvar img = data.result.value.ocratoken.img;\n\t\t\t\tvar url = data.result.value.ocratoken.url;\n\t\t\t\tvar trans = data.result.value.ocratoken.transaction;\n\t\t\t\t$(\'#ocra2_link\').attr("href", url);\n\t\t\t\t$(\'#ocra2_qr_code\').html(img);\n\t\t\t\t$(\'#qr2_activate\').hide();\n\t\t\t\t//$(\'#activationcode\').attr("disabled","disabled");\n\t\t\t\t$(\'#transactionid2\').attr("value", trans);\n\t\t\t\t$(\'#qr2_finish\').show();\n\t\t\t\t$(\'#qr2_confirm1\').show();\n\t\t\t\t$(\'#qr2_confirm2\').show();\n\t\t\t}\n\t\t} else {\n\t\t\talert(activation_fail + " \\n" + data.result.error.message);\n\t\t}\n\t});\n}\n\n\nfunction finishOcra2() {\n\tshow_waiting();\n\tvar trans = $(\'#transactionid2\').val();\n\tvar serial = $(\'#serial2\').val();\n\tvar ocra_check = $(\'#ocra2_check\').val();\n\tvar ocra_finish_ok = $(\'#ocra2_finish_ok\').val();\n\tvar ocra_finish_fail = $(\'#ocra2_finish_fail\').val();\n\n\t$.post("/selfservice/userfinshocra2token", {\n\t\t\'type\' : \'ocra2\',\n\t\t\'serial\' : serial,\n\t\t\'transactionid\' : trans,\n\t\t\'pass\' : ocra_check,\n\t\t\'from\' : \'finishOcra2\',\n\t\t\'session\' : get_selfservice_session()\n\t}, function(data, textStatus, XMLHttpRequest) {\n\t\thide_waiting();\n\n\t\t//console_log(data.result)\n\n\t\tif (data.result.status == true) {\n\t\t\t// The token was successfully initialized and we will display the url\n\t\t\t// if not (false) display an ocra_finish_fail message for retry\n\t\t\tshowTokenlist();\n\t\t\tif (data.result.value.result == false) {\n\t\t\t\talert(ocra_finish_fail);\n\t\t\t} else {\n\t\t\t\talert(String.sprintf(ocra_finish_ok, serial));\n\t\t\t\t$(\'#qr2_completed\').show();\n\t\t\t\t$(\'#qr2_finish\').hide();\n\t\t\t\t//$(\'#ocra_check\').attr("disabled","disabled");\n\t\t\t\t$(\'#ocra2_qr_code\').html(\'<div/>\');\n\t\t\t\t$(\'#qr2_completed\').html(String.sprintf(ocra_finish_ok, serial));\n\t\t\t}\n\t\t} else {\n\t\t\talert("Failed to enroll token!\\n" + data.result.error.message);\n\t\t}\n\t});\n\n}\n\n\n\n\t$(\'#qr2_finish\').hide();\n\t$(\'#qr2_completed\').hide();\n\t$(\'#qr2_confirm1\').hide();\n\t$(\'#qr2_confirm2\').hide();\n\t$(\'#ocra2_check\').removeAttr("disabled");\n\t$(\'#activationcode2\').removeAttr("disabled");\n</script>\n\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''
import logging
from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)

class TestRadiusToken(TestController):

    p = None

    def test_00_create_radius_token(self):
        # The token with the remote PIN
        parameters1 = {
                      "serial"  : "radius1",
                      "type"    : "radius",
                      "otpkey"  : "1234567890123456",
                      "otppin"  : "",
                      "user"    : "remoteuser",
                      "pin"     : "pin",
                      "description" : "RadiusToken1",
                      'radius.server' : 'localhost:18012',
                      'radius.local_checkpin' : 0,
                      'radius.user' : 'user_with_pin',
                      'radius.secret' : 'testing123',
                      }

        # the token with the local PIN
        parameters2 = {
                      "serial"  : "radius2",
                      "type"    : "radius",
                      "otpkey"  : "1234567890123456",
                      "otppin"  : "local",
                      "user"    : "localuser",
                      "pin"     : "pin",
                      "description" : "RadiusToken2",
                      'radius.server' : 'localhost:18012',
                      'radius.local_checkpin' : 1,
                      'radius.user' : 'user_no_pin',
                      'radius.secret' : 'testing123',
                      }

        response = self.app.get(url(controller='admin', action='init'), params=parameters1)
        self.assertTrue('"value": true' in response, response)

        response = self.app.get(url(controller='admin', action='init'), params=parameters2)
        self.assertTrue('"value": true' in response, response)

        response = self.app.get(url(controller='admin', action='set'), params={'serial':'radius2', 'pin':'local'})
        self.assertTrue('"set pin": 1' in response, response)

        response = self.app.get(url(controller='admin', action='set'), params={'serial':'radius1', 'pin':''})
        self.assertTrue('"set pin": 1' in response, response)

    def deleteRadiusToken(self):
        parameters = {
                      "serial"  : "radius1",
                      }

        response = self.app.get(url(controller='admin', action='remove'), params=parameters)

        parameters = {
                      "serial"  : "radius2",
                      }

        response = self.app.get(url(controller='admin', action='remove'), params=parameters)
        log.debug(response)

    def _start_radius_server(self):
        '''
        Start the dummy radius server
        '''
        '''
        We need to start the radius server for every test, since every test instatiates a new TestClass and thus the
        radius server process will not be accessable outside of a test anymore
        '''
        import subprocess

        self.p = subprocess.Popen(["../../tools/dummy_radius_server.py"])

        assert self.p is not None

    def _stop_radius_server(self):
        '''
        stopping the dummy radius server
        '''
        if self.p:
            r = self.p.kill()
            log.debug(r)


    def test_02_check_token_local_pin(self):
        '''
        Checking if token with local PIN works
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "localuser", "pass": "local654321"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        log.error("CKO %s" % parameters)
        self._stop_radius_server()
        self.assertTrue('"value": true' in response, response)


    def test_03_check_token_remote_pin(self):
        '''
        Checking if remote PIN works
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "remoteuser", "pass": "test123456"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        self._stop_radius_server()
        self.assertTrue('"value": true' in response, response)

    def test_04_check_token_local_pin_fail(self):
        '''
        Checking if a missing local PIN will fail
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "localuser", "pass": "654321"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        self._stop_radius_server()

        assert '"value": false' in response

    def test_05_check_token_local_pin_fail2(self):
        '''
        Checking if a wrong local PIN will fail
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "localuser", "pass": "blabla654321"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        self._stop_radius_server()

        assert '"value": false' in response

    def test_06_check_token_remote_pin_fail(self):
        '''
        Checking if a missing remote PIN will fail
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "remoteuser", "pass": "123456"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        self._stop_radius_server()

        assert '"value": false' in response

    def test_06_check_token_remote_pin_fail2(self):
        '''
        Checking if a wrong remote PIN will fail
        '''
        #FIXME dummyradius server is missing
        return
        self._start_radius_server()
        parameters = {"user": "remoteuser", "pass": "abcd123456"}
        response = self.app.get(url(controller='validate', action='check'), params=parameters)
        self._stop_radius_server()

        assert '"value": false' in response


    def test_xx_clean_up(self):
        '''
        Deleting tokens 
        '''
        self.deleteRadiusToken()




#!/usr/bin/env python
# THis is the app to run with uwsgi
import os 
from paste.script.util.logging_config import fileConfig

INIFILE = '/etc/privacyidea/privacyidea.ini'

os.environ['PYTHON_EGG_CACHE'] = '/var/tmp'

fileConfig( INIFILE )
from paste.deploy import loadapp

application = loadapp('config:%s' % INIFILE)



# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
realm handling
'''


from privacyidea.model import Realm, TokenRealm
from privacyidea.model.meta import Session

from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.config import storeConfig
from privacyidea.lib.config import getFromConfig
from privacyidea.lib.log import log_with
from sqlalchemy import func

import logging
log = logging.getLogger(__name__)


@log_with(log)
def createDBRealm(realm):
    '''
        Store Realm in the DB Realm Table. 
        If the realm already exist, we do not need to store it
       
       @param realm: the realm name
       @type  realm: string
       
       @return : if realm is created(True) or already esists(False)
       @rtype  : boolean
    '''

    ret = False
    if not getRealmObject(name=realm):
        log.debug("No realm with name %s exist in database. Creating new" % realm)
        r = Realm(realm)
        r.storeRealm()
        ret = True

    return ret

@log_with(log)
def realm2Objects(realmList):
    ''' 
        convert a list of realm names to a list of realmObjects
        
        @param realmList: list of realnames
        @type  realmList: list
        
        @return: list of realmObjects
        @rtype:  list
    '''

    realmObjList = []
    if realmList is not None:
        for r in realmList:
            realmObj = getRealmObject(name=r)
            if realmObj is not None:
                log.debug("added realm %s to realmObjList" % realmObj)
                realmObjList.append(realmObj)
    return realmObjList

@log_with(log)
def getRealmObject(name=u"", id=0):
    '''
        returns the Realm Object for a given realm name. 
        If the given realm name is not found, it returns "None"
        
        @param name: realmname to be searched
        @type  name: string
        
        @TODO: search by id not implemented, yet
        @param id:   id of the realm object 
        @type  id:   integer
        
        @return : realmObject - the database object
        @rtype  : the sql db object
        
    '''

    log.debug("getting Realm object for name=%s, id=%i" % (name, id))
    realmObj = None
    name = u'' + str(name)
    if (0 == id):
        realmObjects = Session.query(Realm).filter(func.lower(Realm.name) == name.lower())
        if realmObjects.count() > 0:
            realmObj = realmObjects[0]
    return realmObj



@log_with(log)
def getRealms(aRealmName=""):
    '''

        lookup for a defined realm or all realms
    
        @note:  the realms dict is inserted into the privacyIDEA Config object 
        so that a lookup has not to reparse the whole config again
        
        @param aRealmName: a realmname - the realm, that is of interestet, if =="" all realms are returned
        @type  aRealmName: string

        @return:  a dict with realm description like 
        @rtype :  dict : {
                    u'myotherrealm': {'realmname': u'myotherrealm', 
                                    'useridresolver': ['useridresolver.PasswdIdResolver.IdResolver.myOtherRes'], 
                                    'entry': u'privacyidea.useridresolver.group.myotherrealm'}, 
                    u'mydefrealm': {'default': 'true', 
                                    'realmname': u'mydefrealm', 
                                    'useridresolver': ['useridresolver.PasswdIdResolver.IdResolver.myDefRes'], 
                                    'entry': u'privacyidea.useridresolver.group.mydefrealm'}, 
                   u'mymixrealm': {'realmname': u'mymixrealm', 
                                   'useridresolver': ['useridresolver.PasswdIdResolver.IdResolver.myOtherRes', 'useridresolver.PasswdIdResolver.IdResolver.myDefRes'], 
                                   'entry': u'privacyidea.useridresolver.group.mymixrealm'}}
        
    '''
    ret = {}

    config = get_privacyIDEA_config()

    realms = config.getRealms()
    ''' only parse once per session '''
    if realms is None:
        realms = _initalGetRealms()
        config.setRealms(realms)

    ''' check if only one realm is searched '''
    if aRealmName != "" :
        if realms.has_key(aRealmName):
            ret[aRealmName] = realms.get(aRealmName)
    else:
        ret.update(realms)
    return ret

@log_with(log)
def _initalGetRealms():
    '''
       initaly parse all config entries, and extract the realm definition

        @return : a dict with all realm definitions
        @rtype  : dict of definitions
    '''
    Realms = {}
    defRealmConf = "privacyidea.useridresolver"
    realmConf = "privacyidea.useridresolver.group."
    defaultRealmDef = "privacyidea.DefaultRealm"
    defaultRealm = None


    dc = get_privacyIDEA_config()
    for entry in dc:

        if entry.startswith(realmConf):

            #the realm might contain dots "."
            # so take all after the 3rd dot for realm
            r = {}
            realm = entry.split(".", 3)
            theRealm = realm[3].lower()
            r["realmname"] = realm[3]
            r["entry"] = entry

            ##resids          = env.config[entry]
            resids = getFromConfig(entry)
            r["useridresolver"] = resids.split(",")

            Realms[theRealm] = r

        if entry == defRealmConf:
            r = {}

            theRealm = "_default_"
            r["realmname"] = theRealm
            r["entry"] = defRealmConf

            #resids          = env.config[entry]
            resids = getFromConfig(entry)
            r["useridresolver"] = resids.split(",")

            defaultRealm = "_default_"
            Realms[theRealm] = r

        if entry == defaultRealmDef:
            defaultRealm = getFromConfig(defaultRealmDef)

    if defaultRealm is not None:
        _setDefaultRealm(Realms, defaultRealm)

    return Realms

@log_with(log)
def _setDefaultRealm(realms, defaultRealm):
    """
        internal method to set in the realm array the default attribute
        (used by the _initalGetRealms)
        
        @param realms: dict of all realm descriptions
        @type  realms: dict
        @param defaultRealm : name of the default realm
        @type  defaultRelam : string
        
        @return success or not
        @rtype  boolean
    """
    ret = False
    for k in realms:
        '''
            there could be only one default realm
            - all other defaults will be removed  
        '''
        r = realms.get(k)
        if k == defaultRealm.lower():
            r["default"] = "true"
            ret = True
        else:
            if r.has_key("default"):
                del r["default"]
    return ret


@log_with(log)
def isRealmDefined(realm):
    '''
        check, if a realm already exists or not
        
        @param realm: the realm, that should be verified
        @type  realm: string
        
        @return :found or not found
        @rtype  :boolean
    '''
    ret = False
    realms = getRealms();
    if realms.has_key(realm.lower()):
        ret = True
    return ret

@log_with(log)
def setDefaultRealm(defaultRealm):
    """
        set the defualt realm attrbute
        
        @note: verify, if the defualtRealm could be empty :""
        
        @param defaultRealm: the default realm name
        @type  defualtRealm: string
        
        @return:  success or not
        @rtype:   boolean 
    """
    ret = isRealmDefined(defaultRealm)
    if True == ret or defaultRealm == "":
        storeConfig(u"privacyidea.DefaultRealm", defaultRealm);
    return ret

@log_with(log)
def getDefaultRealm():
    """
        return the default realm 
        - lookup in the config for the DefaultRealm key
        
        @return: the realm name
        @rtype : string
    """
    defaultRealmDef = "privacyidea.DefaultRealm"
    defaultRealm = getFromConfig(defaultRealmDef, "")

    if defaultRealm is None or defaultRealm == "":
        log.warning("Serious configuration Issue: no Default Realm defined!")
        defaultRealm = ""

    return defaultRealm.lower()


@log_with(log)
def deleteRealm(realmname):
    '''
        delete the realm from the Database Table with the given name
        
        @param realmname: the to be deleted realm
        @type  realmname: string 
    
    '''
    r = getRealmObject(name=realmname)
    if r is None:
        ''' if no realm is found, we re-try the lowercase name for backward compatibility '''
        r = getRealmObject(name=realmname.lower())
    realmId = 0
    if r is not None:
        realmId = r.id

        if realmId != 0:
            log.debug("Now deleting all realations with realm_id=%i" % realmId)
            Session.query(TokenRealm).filter(TokenRealm.realm_id == realmId).delete()
        Session.delete(r)

    else:
        log.warning("There is no realm object with the name %s to be deleted." % realmname)
        return False
    # now delete all relations, i.e. remove all Tokens from this realm.

    return True




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-17 Fix the empty result problem
#             Cornelius Kölbel, <cornelius@privacyidea.org>
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''This is the BaseClass for audit trails
'''


import logging
log = logging.getLogger(__name__)
from pylons import config
from pylons import tmpl_context as c
from pylons import request
from privacyidea.lib.log import log_with
import socket

import json

from privacyidea.lib.token import getTokenNumResolver


@log_with(log)
def getAuditClass(packageName, className):
    """
        helper method to load the Audit class from a given
        package in literal:

        example:

            getAuditClass("SQLAudit", "Audit")

        check:
            checks, if the log method exists
            if not an error is thrown

"""
    if packageName is None:
        log.error("No suiteable Audit Class found. Working with dummy "
                  "AuditBase class.")
        packageName = "privacyidea.lib.auditmodules"
        className = "AuditBase"

    mod = __import__(packageName, globals(), locals(), [className])
    klass = getattr(mod, className)
    log.debug("klass: %s" % klass)
    if not hasattr(klass, "log"):
        raise NameError("Audit AttributeError: " + packageName + "." +
                        className + " instance has no attribute 'log'")
        return ""
    else:
        return klass


@log_with(log)
def getAudit():
    audit_type = config.get("privacyideaAudit.type")
    audit = getAuditClass(audit_type, "Audit")()
    return audit


def logTokenNum():
    # log the number of the tokens
    c.audit['action_detail'] = "tokennum = %s" % str(getTokenNumResolver())


class AuditBase(object):
    # TODO: Fix: remove the fixed private key
    private = '''-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEArSQlRpXLI+GzARnqOe+XKEv5dy59pefcx/nXt5GGwzAthyiS
lH1oC4VwFgEyh1nvM9pOtQ2sDg5EnpCOTxkGoX+wS9mqpF5S9QhJP5IkrGn4wmIC
NPpCC3ZAzPu8TH2uFzmp8jKzbevtj6s+WodlwFUGVxD55mht2oCcD97elUmNecgQ
TmKU1r9BFt6ZEsDxXUvC1fOWStUwZnj3Heu8Hgte/5BvyBnAaK4QEfga6bKXHi33
80EMTgOYlko1Zl8xJW+QXcAiaF1yqyFqM7FjYVZOZtt0iMy9OsSXw3uT0Te05zaG
qofj9yrtTCuMrf0KxukpgGbwobirh3T+Qnr+pwIDAQABAoIBAHCJTuUrFadT2sp7
cp+HmAMsJpCNmkOMihc80DZTk3koxl7UQznarRbX+3uB+bq5/N0CJyhNI6jbI0TB
Bo5o4MN1wDv81YoSeO8lHJ8COW4LTxHhLDgM9YKHsSTK9p/tDIuyAkEXLULkFzvL
fTLQUJWLbhyHPzbAZ66e61DxdlEnNMEBsQKDKQ51U4TxtHIsYxzThC2sxSSg7keo
lIAp7j913Z7YMSmAJ+eV3A1yfEuxzoxYfNBtFxxeszVIuTjznwcKxs2TWpYxikv8
kmdx654I5UZWsupLRxpJL8L+R1c9J2k9Zeu68WU+1rrjBdnqp3IyxpkPtvWNVXZW
Z84B9zkCgYEA2f56F3H6AZNERIEHGDfGKijvEvhTryuLTdxa8Kt0EbCe1QPgZnBa
bNghEfhuXEIhiBv9V01Au+ljbh1DK7iuThprEIOKzESaPcBTmNYWTUQaaxxlwlDG
ipxjmLZYc+BNFcdAEHxvGfHwfz4GJfhqyUxcnkg2YllItvJn5+DA2yMCgYEAy1PL
G7wUqCTbrb2vfmFMx8MdiSxjuL36Lu7Gx59BiGph4InI57IXSiSP1NWkWIEzCibW
a1X+P9t4iKJ1pTbiSBsntLHNraIeOneUIJY/EFItY+xpj5KDRw+HqDn55sXLCT5n
UabSoTyYYacUTIHAbU8lEW5+Efr7KR2w5uz/+K0CgYEAgEmqQDHrFxI7krT8H9xy
2kzMpTVOyj+t81xCiG/eFqsCTgnB/YcRMAzhKVoyWEjyws72AHKOLgfjY+IEra45
pe0WJNnEzQFyY/TTPZZ/+Wiiw3YqzHgM33W5hx2IYGkX9EEWCp2wJGylQ7yUkbPn
5B70QpHsr2Qrzr5JN8SkulECgYBIeuxSTK+IaOsuegnPIVw/cZxbw8kgmAhRJqkR
jAHOYS3W3wcRIPkQYwwqsKXPLu9E1SdmR9dEaDYFbvRFGtV7IsL6tM8+8CWabfoN
y8FbThAEKMhQd8f4Ut7m5xPgYe3Is8gc4T0AYRto5ChmRXKVBLuQBTVHr5JMy9q9
1wpAWQKBgF0O/coMgJdp983OW7TEK1AgpW19cRVVR564dyZ8F81UwWCzOJqYfSsq
4R7DlPp68IXz7Lg+pCL3T5CweL2ibRTSB2SjukwjGzdeSHbkwI8YEzbH7yIT+Icm
ROxsxyEsfAZEJ5PwTNcARl15QOl4gF2kdFAzlZLz14kioZunMQyL
-----END RSA PRIVATE KEY-----'''
    public = '''-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEArSQlRpXLI+GzARnqOe+X
KEv5dy59pefcx/nXt5GGwzAthyiSlH1oC4VwFgEyh1nvM9pOtQ2sDg5EnpCOTxkG
oX+wS9mqpF5S9QhJP5IkrGn4wmICNPpCC3ZAzPu8TH2uFzmp8jKzbevtj6s+Wodl
wFUGVxD55mht2oCcD97elUmNecgQTmKU1r9BFt6ZEsDxXUvC1fOWStUwZnj3Heu8
Hgte/5BvyBnAaK4QEfga6bKXHi3380EMTgOYlko1Zl8xJW+QXcAiaF1yqyFqM7Fj
YVZOZtt0iMy9OsSXw3uT0Te05zaGqofj9yrtTCuMrf0KxukpgGbwobirh3T+Qnr+
pwIDAQAB
-----END PUBLIC KEY-----'''

    def __init__(self):
        self.name = "AuditBase"

    @log_with(log)
    def initialize(self):
        # defaults
        c.audit = {'action_detail': '',
                   'info': '',
                   'log_level': 'INFO',
                   'administrator': '',
                   'value': '',
                   'key': '',
                   'serial': '',
                   'token_type': '',
                   'clearance_level': 0,
                   'privacyidea_server': socket.gethostname(),
                   'realm': '',
                   'user': '',
                   'client': ''
                   }
        controller = request.environ['pylons.routes_dict']['controller']
        action = request.environ['pylons.routes_dict']['action']
        c.audit['action'] = "%s/%s" % (controller, action)

    @log_with(log)
    def readKeys(self):
        priv = config.get("privacyideaAudit.key.private")
        pub = config.get("privacyideaAudit.key.public")
        try:
            f = open(priv, "r")
            self.private = f.read()
            f.close()
        except Exception as e:
            log.error("Error reading private key %s: (%r)" % (priv, e))

        try:
            f = open(pub, "r")
            self.public = f.read()
            f.close()
        except Exception as e:
            log.error("Error reading public key %s: (%r)" % (pub, e))

    def getAuditId(self):
        return self.name

    def getTotal(self, param, AND=True, display_error=True):
        '''
        This method returns the total number of audit entries
        in the audit store
        '''
        return None

    @log_with(log)
    def log(self, param):
        '''
        This method is used to log the data.
        It should hash the data and do a hash chain and sign the data
        '''
        pass

    def initialize_log(self, param):
        '''
        This method initialized the log state.
        The fact, that the log state was initialized, also needs to be logged.
        Therefor the same params are passed as i the log method.
        '''
        pass

    def set(self):
        '''
        This function could be used to set certain things like the signing key.
        But maybe it should only be read from privacyidea.ini?
        '''
        pass

    def search(self, param, AND=True, display_error=True, rp_dict=None):
        '''
        This function is used to search audit events.

        param:
            Search parameters can be passed.

        return:
            A list of dictionaries is return.
            Each list element denotes an audit event.
            
        This function is deprecated.
        '''
        if rp_dict is None:
            rp_dict = {}
        result = [{}]
        return result
    
    def searchQuery(self, search_dict, rp_dict):
        '''
        This function returns the audit log as an iterator on the result
        '''
        return None
    
    def audit_entry_to_dict(self, audit_entry):
        '''
        If the searchQuery returns an iteretor with elements that are not a
        dictionary, the audit module needs
        to provide this function, to convert the audit entry to a dictionary.
        '''
        return {}


@log_with(log)
def search(param, user=None, columns=None):

    audit = getAudit()
    
    search_dict = {}

    if "query" in param:
        if "extsearch" == param['qtype']:
            # search patterns are delimitered with ;
            search_list = param['query'].split(";")
            for s in search_list:
                log.debug(s)
                key, _e, value = s.partition("=")
                key = key.strip()
                value = value.strip()
                search_dict[key] = value
            log.debug(search_dict)

        else:
            search_dict[param['qtype']] = param["query"]
    else:
        for k, v in param.items():
            search_dict[k] = v

    log.debug("search_dict: %s" % search_dict)

    rp_dict = {}
    rp_dict['page'] = param.get('page')
    page = 1
    if param.get('page'):
        page = param.get('page')

    rp_dict['rp'] = param.get('rp')
    rp_dict['sortname'] = param.get('sortname')
    rp_dict['sortorder'] = param.get('sortorder')
    log.debug("[rp_dict: %s" % rp_dict)
    if user:
        search_dict['user'] = user.login
        search_dict['realm'] = user.realm

    result = audit.search(search_dict, rp_dict=rp_dict)

    lines = []
    if columns:
        # In this case we have only a limited list of columns, like in
        # the selfservice portal
        for a in result:
            if "number" in a:
                cell = []
                for c in columns:
                    cell.append(a.get(c))
                lines.append({'id': a['number'],
                              'cell': cell
                              })
    else:
        # Here we use all columns, that exist
        for a in result:
            if "number" in a:
                lines.append({'id': a['number'],
                              'cell': [a.get('number', ''),
                                       a.get('date', ''),
                                       a.get('sig_check', ''),
                                       a.get('missing_line', ''),
                                       a.get('action', ''),
                                       a.get('success', ''),
                                       a.get('serial', ''),
                                       a.get('token_type', ''),
                                       a.get('user', ''),
                                       a.get('realm', ''),
                                       a.get('administrator', ''),
                                       a.get('action_detail', ''),
                                       a.get('info', ''),
                                       a.get('privacyidea_server', ''),
                                       a.get('client', ''),
                                       a.get('log_level', ''),
                                       a.get('clearance_level', ''),
                                       ]
                              }
                             )
    # get the complete number of audit logs
    total = audit.getTotal(search_dict)

    return lines, total, page


class AuditIterator(object):

    def __init__(self, param, user=None, columns=None):
        self.param = param
        self.user = user
        self.columns = columns
        self.count = 0
        self.last = None
        self.page = 1
        self.audit = None
        self.iter = None
        self.headers = False
        self.search_dict = {}
        self.audit = getAudit()

    def __iter__(self):
        """
        start iteration
        """
        search_dict = {}
        param = self.param

        if 'headers' in param:
            self.headers = True
            del param['headers']

        if "query" in param:
            if "extsearch" == self.param['qtype']:
                # search patterns are delimited with ;
                search_list = param['query'].split(";")
                for s in search_list:
                    log.debug(s)
                    key, _e, value = s.partition("=")
                    key = key.strip()
                    value = value.strip()
                    search_dict[key] = value
                log.debug(search_dict)

            else:
                search_dict[param['qtype']] = param["query"]
        else:
            for k, v in param.items():
                search_dict[k] = v

        log.debug("search_dict: %s" % search_dict)

        rp_dict = {}
        page = param.get('page', None) or None
        if page is not None:
            rp_dict['page'] = param.get('page')
        self.page = param.get('page', 1)

        rp_dict['rp'] = param.get('rp', '15') or '15'

        rp_dict['sortname'] = param.get('sortname')
        rp_dict['sortorder'] = param.get('sortorder')
        log.debug("rp_dict: %s" % rp_dict)

        if self.user:
            search_dict['user'] = self.user.login
            search_dict['realm'] = self.user.realm

        # fetch the query iterator
        self.iter = self.audit.searchQuery(search_dict, rp_dict=rp_dict)
        self.search_dict = search_dict

        return self

    def next(self):
        """
        call the sql alchemy row/result iterator
        :return: row as a dict
        """

        entry = {}
        a = self.iter.next()

        if type(a) != dict:
            # convert table data to dict!
            a = self.audit.audit_entry_to_dict(a)

        columns = self.columns
        if columns:
            # In this case we have only a limited list of columns, like in
            # the selfservice portal
            if 'number' in a:
                cell = []
                for c in columns:
                    cell.append(a.get(c))

                # Fixme: why is this not used?
                _lentry = {'id': a['number'],
                           'cell': cell}
        else:
            # Here we use all columns, that exist
            if 'number' in a:
                entry = {'id': a['number'],
                         'cell': [a.get('number', ''),
                                  a.get('date', ''),
                                  a.get('sig_check', ''),
                                  a.get('missing_line', ''),
                                  a.get('action', ''),
                                  a.get('success', ''),
                                  a.get('serial', ''),
                                  a.get('token_type', ''),
                                  a.get('user', ''),
                                  a.get('realm', ''),
                                  a.get('administrator', ''),
                                  a.get('action_detail', ''),
                                  a.get('info', ''),
                                  a.get('privacyidea_server', ''),
                                  a.get('client', ''),
                                  a.get('log_level', ''),
                                  a.get('clearance_level', ''),
                                  ],
                         }
                if self.headers is True:
                    entry['data'] = ['number',
                                     'date',
                                     'sig_check',
                                     'missing_line',
                                     'action',
                                     'success',
                                     'serial',
                                     'token_type',
                                     'user',
                                     'realm',
                                     'administrator',
                                     'action_detail',
                                     'info',
                                     'privacyidea_server',
                                     'client',
                                     'log_level',
                                     'clearance_level',
                                     ]

            return entry


class JSONAuditIterator(AuditIterator):
    """
    default audit output generator in json format
    """
    def __init__(self, param, user=None, columns=None):
        self.parent = super(JSONAuditIterator, self)
        self.parent.__init__(param, user=user, columns=columns)
        self.count = 0
        self.last = None

    def next(self):
        """
        iterator callback for the next chunk of data

        :return: returns a string representing the data row
        """
        beginning = ""
        closing = ""
        cell = ""
        if self.last is not None:
            raise self.last

        if self.count == 0:
            beginning = ('{ "page": %d, "rows": [') % int(self.page)
            self.count = self.count + 1

        try:
            entry = self.parent.next()
        except Exception as exx:
            self.last = exx
            # get the complete number of audit logs
            total = self.audit.getTotal(self.search_dict)
            closing = '{} ], "total": %d }' % int(total)
            if self.count == 1:
                # There was no other entry and we just return an empty list
                return beginning + closing
            else:
                return closing

        entry_s = json.dumps(entry, indent=3)

        if self.count > 0:
            self.count = self.count + 1
            cell = entry_s + ", "
        return beginning + cell


class CSVAuditIterator(AuditIterator):
    """
    create cvs output by iterating over result
    """

    def __init__(self, param, user=None, columns=None):
        self.parent = super(CSVAuditIterator, self)
        self.parent.__init__(param, user=user, columns=columns)
        self.count = 0
        self.last = None
        self.delimiter = param.get('delimiter', ',') or ','

    def next(self):
        """
        iterator callback for the next chunk of data

        :return: returns a string representing the data row
        """
        result = ""

        if self.last is not None:
            raise self.last

        try:
            entry = self.parent.next()
        except Exception as exx:
            self.last = exx
            return "\n"

        if self.headers is True and self.count == 0:
            # Do the header
            row = entry.get('data', [])
            r_str = json.dumps(row)[1:-1]

            result += r_str
            result += "\n"

        row = []
        raw_row = entry.get('cell', [])

        # we must escape some dump entries, which destroy the
        # import of the csv data - like SMSProviderConfig 8-(
        for row_entry in raw_row:
            if type(row_entry) in (str, unicode):
                row_entry = row_entry.replace('\"', "'")
            row.append(row_entry)

        r_str = json.dumps(row)[1:-1]
        result += r_str
        result += "\n"

        self.count = self.count + 1

        return result

# ##eof#######################################################################



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
This file is part of the privacyidea service
This is the controller for the openid service
'''


ASSOC_EXPIRES_IN = 3600
COOKIE_NAME = "privacyidea_openid"


import logging

import webob
import traceback



from pylons import tmpl_context as c
from pylons import request, response
from pylons.controllers.util import redirect
from pylons import config
from pylons import url as url

from privacyidea.lib.base import BaseController

from privacyidea.lib.util import  getParam
from privacyidea.weblib.util import get_client
from privacyidea.lib.util import  is_valid_fqdn
from privacyidea.lib.user import  getUserFromParam
from privacyidea.lib.realm import  getDefaultRealm
from privacyidea.lib.realm import getRealms
from privacyidea.lib.user     import getRealmBox
from privacyidea.lib.util    import get_version
from privacyidea.lib.util    import get_copyright_info

from privacyidea.lib.policy import PolicyException
from pylons.templating import render_mako as render
from webob.exc import HTTPBadRequest


from privacyidea.lib.token import checkUserPass
from privacyidea.lib.reply import sendError
from privacyidea.model.meta import Session

from privacyidea.lib.openid import IdResMessage
from privacyidea.lib.openid import create_association, check_authentication

from privacyidea.lib.openid import OPENID_2_0_TYPE
from privacyidea.lib.openid import OPENID_1_0_TYPE
from privacyidea.lib.log import log_with

from urllib import urlencode


optional = True
required = False

log = logging.getLogger(__name__)

class OpenidController(BaseController):

    '''
    this is the controller for doing the openid stuff

        https://server/openid/<functionname>

    '''
    BASEURL = "https://privacyideaserver"
    COOKIE_EXPIRE = 3600

    @log_with(log)
    def __before__(self, action, **params):
        valid_request = False
        try:
            for key, value in params.items():
                log.debug("%r:%r]" % (key, value))

            c.audit['client'] = get_client()

            self.storage = config.get('openid_sql')

            getCookieExpire = int(config.get("privacyideaOpenID.CookieExpire", -1))

            self.COOKIE_EXPIRE = 3600
            if getCookieExpire >= 0:
                self.COOKIE_EXPIRE = getCookieExpire

            c.logged_in = False
            c.login = ""
            c.version = get_version()
            c.licenseinfo = get_copyright_info()

            http_host = request.environ.get("HTTP_HOST")
            log.debug("Doing openid request from host %s" % http_host)
            if not is_valid_fqdn(http_host, split_port=True):
                err = "Bad hostname: %s" % http_host
                self.audit.log(c.audit)
                c.audit["action_detail"] = err
                log.error(err)
                raise HTTPBadRequest(err)

            self.BASEURL = request.environ.get("wsgi.url_scheme") + "://" + http_host

            # check if the browser is logged in
            login = request.cookies.get(COOKIE_NAME)

            if login:
                c.logged_in = True

            ## default return for the __before__ and __after__
            valid_request = True
            return response

        except PolicyException as pex:
            log.error("%r policy exception %r" % (action, pex))
            log.error(traceback.format_exc())
            return sendError(response, pex, context='before')

        except webob.exc.HTTPUnauthorized as acc:
            ## the exception, when an abort() is called if forwarded
            log.error("%r webob.exception %r" % (action, acc))
            log.error(traceback.format_exc())
            raise acc

        except Exception as exx:
            log.error("%r exception %r" % (action, exx))
            log.error(traceback.format_exc())
            return sendError(response, exx, context='before')

        finally:
            if valid_request is False:
                self.storage.session.rollback()
                self.storage.session.close()


    @log_with(log)
    def __after__(self, action, **params):
        try:
            self.audit.log(c.audit)
            self.storage.session.commit()
            ## default return for the __before__ and __after__
            return response

        except Exception as exx:
            log.error("%r" % (exx))
            log.error(traceback.format_exc())
            self.storage.session.rollback()
            return sendError(response, exx, context='after')

        finally:
            self.storage.session.close()


    def id(self):
        '''
        This method is used by the consumer to authenticate like this:
        https://server/openid/id/<user>

        The URL has to return this one in the html head:
        <link rel="openid.server" href="http://localhost.localdomain:8011/openidserver">
        <meta http-equiv="x-xrds-location" content="http://localhost.localdomain:8011/yadis/corny">

        The request flow is:
            -> GET /openid/id
            -> GET /openid/yadis
            -> POST /openid/openidserver -> assocication
            -> POST /openid/openidserver -> checkid setup

        '''
        user = request.environ['pylons.routes_dict'].get('id')
        log.debug("requesting access for user %s" % user)

        baseurl = self.BASEURL
        baseyadis = baseurl + "/openid/yadis/"
        endpoint = baseurl + "/openid/openidserver"

        response.content_type = 'text/html'
        head = '''\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
  <meta http-equiv="Content-type" content="text/html;charset=UTF-8" />
  <link rel="openid.server" href="%s" />
  <meta http-equiv="x-xrds-location" content="%s%s" />
</head>
<body>This is used to issue the user names</body>
</html>''' % (endpoint, baseyadis, user)
        return head


    def yadis(self):
        user = request.environ['pylons.routes_dict'].get('id')
        response.content_type = 'application/xrds+xml'

        endpoint_url = self.BASEURL + "/openid/openidserver"
        user_url = self.BASEURL + "/openid/id/%s" % user

        body = """\
<?xml version="1.0" encoding="UTF-8"?>
<xrds:XRDS xmlns:xrds="xri://$xrds" xmlns="xri://$xrd*($v*2.0)">
  <XRD>
    <Service priority="0">
      <Type>%s</Type>
      <Type>%s</Type>
      <URI>%s</URI>
      <LocalID>%s</LocalID>
    </Service>
  </XRD>
</xrds:XRDS>
""" % (OPENID_2_0_TYPE, OPENID_1_0_TYPE, endpoint_url, user_url)

        return body

    @log_with(log)
    def openidserver(self, action, **params):
        '''
        This is the so called server endpoint, that decides, if the user is authenticated or not.
        and returns to the given "openid." either directly or after authenticating the user
        openid.claimed_id.
        '''

        req_method = request.environ.get('REQUEST_METHOD')

        if "POST" == req_method:
            params = request.environ.get('webob._parsed_post_vars')
            params = params[0]
        else:
            # we got a GET request
            params = request.params

        # distpatching the request depending on the mode
        if 'openid.mode' not in params:
            raise HTTPBadRequest('Missing "openid.mode"')

        mode = params.get('openid.mode')
        log.debug("openid.mode=%s" % mode)

        if 'associate' == mode:
            return self.associate(params)

        # mandatory fields for other actions
        for field in ('openid.identity',):  #, 'openid.'):
            if field not in params:
                raise HTTPBadRequest('Missing "%s"' % field)

        if 'check_authentication' == mode:
            return self.check_authentication(params)

        elif mode in ('checkid_setup', 'checkid_immediate'):

            authenticated = False
            c.login, token = self._split_cookie()
            if "" != c.login:
                # what user wants to login?
                rest, claimed_user = params.get("openid.claimed_id").rsplit("/", 1)
                stored_token = self.storage.get_user_token(c.login)

                log.debug("checking authenticated? %s=%s, %s=%s" %
                            (stored_token, token, c.login, claimed_user))

                if stored_token == token and self._compare_users(c.login, claimed_user):
                    authenticated = True

            if not authenticated:
                # Not logged in!
                redirect_to = self.BASEURL + "/openid/openidserver"
                openid_params = urlencode(params)
                redirect("/openid/login?%s&%s" % (urlencode({ "redirect_to" : redirect_to }), openid_params))
            else:
                return self.checkid_setup(params)

        # other modes are ignored
        raise HTTPBadRequest('"%s" mode not supported' % mode)


    def checkid_setup(self, param):
        '''
        This function is called, when the user needs to verify that he is willing to
        authenticate for a relying party
        '''
        params = {}
        params.update(param)

        HOST = self.BASEURL + "/openid/openidserver"
        message = IdResMessage(self.storage, HOST, 3600, **params)
        # signing it
        message.sign()

        # for checking trusted roots
        user, token = self._split_cookie()
        redirect_token = message.store_redirect()
        _url, site, handle = self.storage.get_redirect(redirect_token)
        trusted_roots = self.storage.get_trusted_roots(user)

        # was it called by the identity plugin ?
        if 'X-Identity' in request.headers:
            # storing the site in allowed sites
            message.store_site()
            # redirecting
            redirect(message.get_url())

        elif site in trusted_roots:
            login, token = self._split_cookie()
            user = self.storage.get_user_by_token(token)
            c.audit['user'], c.audit['realm'] = user.split('@', 2)
            c.audit['success'] = True
            c.audit['action_detail'] = site
            c.audit['info'] = "site found in trusted root"
            # automatic validate, i.e.
            # the user gets redirected to the relying party
            redirect_to = message.get_url()
            redirect(redirect_to)

        else:
            # if not, we store the redirect url and display
            # a manual screen the user needs to validate
            c.identity = message.identity
            c.redirect_token = redirect_token
            c.rely_party = message.site
            return render('/openid/check_setup.mako')


    @log_with(log)
    def checkid_submit(self, action, **params):
        '''
        This is called when the user accepts - hit the submit button - that he will login to the consumer
        '''
        param = request.params
        redirect_token = getParam(param, "redirect_token", required)
        verify_always = getParam(param, "verify_always", optional)
        r_url, site, handle = self.storage.get_redirect(redirect_token)
        self.storage.add_site(site, handle)

        # The user checked the box, that he wants not be bothered again in the future
        # the relying party will be added to the trusted root
        login, token = self._split_cookie()
        user = self.storage.get_user_by_token(token)
        c.audit['user'], c.audit['realm'] = user.split('@', 2)
        c.audit['success'] = True
        c.audit['action_detail'] = site

        if "always" == verify_always:
            log.debug("putting into trusted root: %s, %s" % (site, handle))
            if "" != user:
                self.storage.add_trusted_root(user, site)

        log.debug("redirecting to %s" % r_url)
        redirect(r_url)

    def check_authentication(self, params):
        res = check_authentication(**params)
        response.status = 200
        response.content_type = "text/plain"
        return res


    def associate(self, params):
        '''
        This sets up a association (encryption key) bewtween the ID Provider and the consumer
        '''
        expires_in = ASSOC_EXPIRES_IN
        res = create_association(self.storage, expires_in, **params)
        response.status = 200
        response.content_type = "text/plain"
        return res


######### Auth stuff #################################################
    def logout(self):
        '''
        This action deletes the cookie and redirects to the
        /openid/status to show the login status

        If the logout is called in the context of an openid authentication,
        the user is already logged in as a different user. In this case we
        forward to the /openid/login page after the logout was made.

        Another option for the openid authentication context would be to
        redirect to the return_to url by setting
            redirect_to = params["openid.return_to"]
            p["openid.mode"] = "setup_needed"
        which advises the openid relying party to restart the login process.
        '''

        response.delete_cookie(COOKIE_NAME)

        params = {}
        params.update(request.params)
        p = {}

        ## are we are called during an openid auth request?
        if "openid.return_to" in params:
            redirect_to = "/openid/login"
            p.update(params)
            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(p))))

        else:
            redirect_to = "/openid/status"
            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(p))))

        redirect(do_redirect)

    def login(self):
        '''
        This is the redirect of the first template
        '''
        param = request.params

        c.defaultRealm = getDefaultRealm()
        c.p = {}
        c.user = ""
        c.title = "privacyIDEA OpenID Service"

        for k in param:
            c.p[k] = param[k]
            if "openid.claimed_id" == k:
                c.user = param[k].rsplit("/", 1)[1]

        ## if we have already a cookie but
        ## a difference between login and cookie user
        ## we show (via  /status) that he is already logged in
        ## and that he first must log out
        cookie = request.cookies.get(COOKIE_NAME)
        if cookie is not None:
            cookie_user, token = cookie.split(":")

            if cookie_user != c.user:
                c.login = cookie_user
                c.message = ("Before logging in as >%s< you have to log out."
                             % (c.user))

                return render("/openid/status.mako")

        return render('/openid/login.mako')

    def status(self):
        '''
        This shows the login status.
        '''
        param = {}
        param.update(request.params)

        cookie = request.cookies.get(COOKIE_NAME)
        if cookie is not None:
            c.login, token = cookie.split(":")

        if "message" in param:
            c.message = param.get("message")

        return render("/openid/status.mako")


    @log_with(log)
    def check(self, action, **params):
        '''
        This function is used to login

        method:
            openid/check

        arguments:
            user     - user to login
            realm    - in which realm the user should login
            pass     - password

        returns:
            JSON response
        '''
        ok = False
        param = {}
        do_redirect = None
        message = None

        try:
            param.update(request.params)

            same_user = True
            passw = getParam(param, "pass", optional)

            ## getUserFromParam will return default realm if no realm is
            ## provided via @ append or extra parameter realm
            ## if the provided realm does not exist, the realm is left empty
            user = getUserFromParam(param, optional)

            ## if the requested user has a realm specified (via @realm append)
            ## and this is not the same as the user from getUserFromParam
            ## the requested user is not a valid one!
            p_user = param.get('user', '')
            if "@" in p_user:
                if p_user != "%s@%s" % (user.login, user.realm):
                    same_user = False

            c.audit['user'] = user.login
            c.audit['realm'] = user.realm or getDefaultRealm()

            if same_user is True:
                (ok, opt) = checkUserPass(user, passw)

            c.audit['success'] = ok

            if ok:
                ## if the user authenticated successfully we need to set the cookie aka
                ## the ticket and we need to remember this ticket.
                user = "%s@%s" % (user.login, c.audit['realm'])
                log.debug("user=%s" % user)
                token = self.storage.set_user_token(user, expire=self.COOKIE_EXPIRE)
                log.debug("token=%s" % token)
                cookie = "%s:%s" % (user, token)
                log.debug("cookie=%s" % cookie)
                response.set_cookie(COOKIE_NAME, cookie, max_age=self.COOKIE_EXPIRE)
            else:
                message = "Your login attempt was not successful!"

            Session.commit()
            # Only if we logged in successfully we redirect to the original
            # page (Servive Provider). Otherwise we will redirect to the
            # status page

            p = {}
            redirect_to = getParam(param, "redirect_to", optional)
            if redirect_to and ok:
                p = {}
                for k in  [ 'openid.return_to', "openid.realm", "openid.ns", "openid.claimed_id", "openid.mode",
                            "openid.identity" ]:
                    p[k] = param[k]
            else:
                if message is not None:
                    p["message"] = message
                redirect_to = "/openid/status"

            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(p))))

        except Exception as exx:
            log.error("openid/check failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, "openid/check failed: %r" % exx, 0)

        finally:
            Session.close()

        if do_redirect:
            log.debug("now redirecting to %s" % do_redirect)
            redirect(do_redirect)

    def _compare_users(self, u1, u2):
        realm = getDefaultRealm()
        if len(u1.split('@')) == 1:
            u1 = "%s@%s" % (u1, realm)
        if len(u2.split('@')) == 1:
            u2 = "%s@%s" % (u2, realm)
        log.debug("%s == %s?" % (u1, u2))
        return u1 == u2


    def _split_cookie(self):
        login = ""
        token = ""
        cookie = request.cookies.get(COOKIE_NAME)
        if cookie is not None:
            login, token = cookie.split(":")
        return login, token

#eof###########################################################################
